@@ -31,7 +31,7 @@ async def update_attachments(
     """
     result = dict(comment=(), result=True, ret=None)
     already_attached_vpc_id = (
-        before["Attachments"][0]["VpcId"] if before["Attachments"] else None
+        before["attachments"][0]["VpcId"] if "attachments" in before else None
     )
     if vpc_id is None:
         result["ret"] = {"updated_vpc_id": [already_attached_vpc_id]}
@@ -83,7 +83,7 @@ async def update_attachments(
         else:
             try:
                 # When vpc_id is empty, detach from vpc if already attached
-                if before["Attachments"]:
+                if "attachments" in before:
                     ret = await hub.exec.boto3.client.ec2.detach_internet_gateway(
                         ctx,
                         InternetGatewayId=internet_gateway_name,
