@@ -32,8 +32,10 @@ async def update_transit_gateway_vpc_attachment(
 
     """
     result = dict(comment=(), result=True, ret=None)
-    if set(old_subnets) == set(new_subnets) and old_options == new_options:
-        result["comment"] = (f"no changes to update",)
+    if set(old_subnets) == set(new_subnets) and (
+        new_options is None
+        or hub.tool.aws.state_comparison_utils.compare_dicts(new_options, old_options)
+    ):
         return result
 
     subnets_to_add = list(set(new_subnets).difference(old_subnets))
@@ -55,9 +57,6 @@ async def update_transit_gateway_vpc_attachment(
         if not ret["result"]:
             result["comment"] = result["comment"] + ret["comment"]
             return result
-        result["comment"] = result["comment"] + (
-            f"Updated transit gateway vpc attachment {transit_gateway_vpc_attachment_id}",
-        )
     result["ret"] = {
         "subnet_ids": new_subnets,
         "options": new_options,
