@@ -1,16 +1,15 @@
+import asyncio
 import copy
-import time
 from typing import Any
 from typing import Dict
-from typing import List
 
 
 async def update_key_tags(
     hub,
     ctx,
     key_id: str,
-    old_tags: List[Dict[str, Any]],
-    new_tags: List[Dict[str, Any]],
+    old_tags: Dict[str, Any],
+    new_tags: Dict[str, Any],
 ):
     """
 
@@ -18,33 +17,28 @@ async def update_key_tags(
         hub:
         ctx:
         key_id: aws kms key id
-        old_tags: list of old tags
-        new_tags: list of new tags
+        old_tags: dict of old tags
+        new_tags: dict of new tags
 
     Returns:
-        {"result": True|False, "comment": Tuple, "ret": "Tags after update"}
+        {"result": True|False, "comment": Tuple, "ret": "dict of tags after update"}
 
     """
     result = dict(comment=(), result=True, ret=None)
 
-    tags_to_add = list()
-    old_tags_map = {tag.get("TagKey"): tag for tag in old_tags}
-    tags_result = copy.deepcopy(old_tags_map)
-    for tag in new_tags:
-        if tag.get("TagKey") in old_tags_map:
-            # Make sure the key and value are the same before deleting
-            if tag.get("TagValue") == old_tags_map.get(tag.get("TagKey")).get(
-                "KeyValue"
-            ):
-                del old_tags_map[tag.get("TagKey")]
-            else:
-                tags_to_add.append(tag)
-        else:
-            tags_to_add.append(tag)
-    tags_to_remove = [tag.get("TagKey") for tag in old_tags_map.values()]
+    tags_to_add = {}
+    tags_to_remove = {}
+    if new_tags is not None:
+        tags_to_remove, tags_to_add = hub.tool.aws.tag_utils.diff_tags_dict(
+            old_tags=old_tags, new_tags=new_tags
+        )
+    if (not tags_to_remove) and (not tags_to_add):
+        result["ret"] = copy.deepcopy(old_tags if old_tags else {})
+        return result
     if tags_to_remove and not ctx.get("test", False):
+        tag_keys = list(tags_to_remove.keys())
         delete_ret = await hub.exec.boto3.client.kms.untag_resource(
-            ctx, KeyId=key_id, TagKeys=tags_to_remove
+            ctx, KeyId=key_id, TagKeys=tag_keys
         )
         if not delete_ret["result"]:
             result["comment"] = delete_ret["comment"]
@@ -52,22 +46,24 @@ async def update_key_tags(
             return result
     if tags_to_add and not ctx.get("test", False):
         add_ret = await hub.exec.boto3.client.kms.tag_resource(
-            ctx, KeyId=key_id, Tags=tags_to_add
+            ctx,
+            KeyId=key_id,
+            Tags=hub.tool.aws.tag_utils.convert_tag_dict_to_list_tagkey(
+                tags=tags_to_add
+            ),
         )
         if not add_ret["result"]:
             result["comment"] = add_ret["comment"]
             result["result"] = False
             return result
-    for key in tags_to_remove:
-        tags_result.pop(key)
-    result["ret"] = {"tags": list(tags_result.values()) + tags_to_add}
+    result["ret"] = new_tags
     if ctx.get("test", False):
-        result["comment"] = (
-            f"Would update tags: Add [{tags_to_add}] Remove [{tags_to_remove}]",
+        result["comment"] = hub.tool.aws.comment_utils.would_update_tags_comment(
+            tags_to_remove=tags_to_remove, tags_to_add=tags_to_add
         )
     else:
-        result["comment"] = (
-            f"Updated tags: Added [{tags_to_add}] Removed [{tags_to_remove}]",
+        result["comment"] = hub.tool.aws.comment_utils.update_tags_comment(
+            tags_to_remove=tags_to_remove, tags_to_add=tags_to_add
         )
     return result
 
@@ -100,7 +96,7 @@ async def wait_for_updates(
         hub.log.info(
             f"Waiting for key {resource_id} updates: '{updates.keys()}' for the {count} time"
         )
-        time.sleep(delay)
+        await asyncio.sleep(delay)
 
         raw_key = await hub.exec.boto3.client.kms.describe_key(ctx, KeyId=resource_id)
         key = await hub.tool.aws.kms.conversion_utils.convert_raw_key_to_present(
