@@ -4,6 +4,8 @@ hub.exec.boto3.client.acm.request_certificate
 hub.exec.boto3.client.acm.describe_certificate
 """
 import copy
+from dataclasses import field
+from dataclasses import make_dataclass
 from typing import Any
 from typing import Dict
 from typing import List
@@ -75,13 +77,59 @@ async def present(
     certificate_chain: str = None,
     domain_name: str = None,
     validation_method: str = None,
-    subject_alternative_names: List = None,
+    subject_alternative_names: List[str] = None,
     idempotency_token: str = None,
-    domain_validation_options: List = None,
-    options: {} = None,
+    domain_validation_options: List[
+        make_dataclass(
+            "DomainValidationOption",
+            [
+                ("domain_name", str),
+                ("validation_domain", str),
+                ("validation_method", str, field(default=None)),
+                (
+                    "resource_record",
+                    make_dataclass(
+                        "ResourceRecord",
+                        [
+                            ("Name", str, field(default=None)),
+                            ("Type", str, field(default=None)),
+                            ("Value", str, field(default=None)),
+                        ],
+                    ),
+                    field(default=None),
+                ),
+                ("validation_status", str, field(default=None)),
+            ],
+        )
+    ] = None,
+    options: make_dataclass(
+        "CertificateOptions",
+        [("CertificateTransparencyLoggingPreference", str, field(default=None))],
+    ) = None,
     certificate_authority_arn: str = None,
-    timeout: Dict = None,
-    tags: List = None,
+    timeout: make_dataclass(
+        "Timeout",
+        [
+            (
+                "create",
+                make_dataclass(
+                    "CreateTimeout",
+                    [
+                        ("delay", int, field(default=0)),
+                        ("max_attempts", int, field(default=0)),
+                    ],
+                ),
+                field(default=None),
+            )
+        ],
+    ) = None,
+    tags: Dict[str, Any]
+    or List[
+        make_dataclass(
+            "Tag",
+            [("Key", str, field(default=None)), ("Value", str, field(default=None))],
+        )
+    ] = None,
 ) -> Dict[str, Any]:
     r"""
 
@@ -113,19 +161,49 @@ async def present(
             extension of the ACM certificate.,
         idempotency_token(Text,Optional): Customer chosen string that can be used to distinguish between calls to
             RequestCertificate .,
-        domain_validation_options(List, optional): The domain name that you want ACM to use to send you emails so that
-            you can validate domain ownership.,
-        options(Dict): to specify whether to add the certificate to a certificate transparency log. Certificate
-            transparency makes it possible to detect SSL/TLS certificates that have been mistakenly or maliciously issued.,
+        domain_validation_options(List[Dict[str, Any]], optional): The domain name that you want ACM to use to send you emails so that
+            you can validate domain ownership. Defaults to None.
+            * domain_name (str): A fully qualified domain name (FQDN) in the certificate request.
+            * validation_domain (str): The domain name that you want ACM to use to send you validation emails. This domain name is the
+                suffix of the email addresses that you want ACM to use. This must be the same as the DomainName
+                value or a superdomain of the DomainName value. For example, if you request a certificate for
+                testing.example.com, you can specify example.com for this value. In that case, ACM sends domain
+                validation emails to the following five addresses:   admin@example.com
+                administrator@example.com   hostmaster@example.com   postmaster@example.com
+                webmaster@example.com
+            * validation_domain (str): The domain name that you want ACM to use to send you validation emails. This domain name is the
+                suffix of the email addresses that you want ACM to use. This must be the same as the DomainName
+                value or a superdomain of the DomainName value. For example, if you request a certificate for
+                testing.example.com, you can specify example.com for this value. In that case, ACM sends domain
+                validation emails to the following five addresses:   admin@example.com
+                administrator@example.com   hostmaster@example.com   postmaster@example.com
+                webmaster@example.com
+            * validation_method (str): Specifies the domain validation method.
+            * resource_record (Dict[str, Any]): Contains the CNAME record that you add to your DNS database for domain validation.
+                * Name (str): The name of the DNS record to create in your domain. This is supplied by ACM.
+                * Type (str): The type of DNS record. Currently this can be CNAME.
+                * Value (str): The value of the CNAME record to add to your DNS database. This is supplied by ACM.
+            * validation_status (str): The validation status of the domain name.
+                This can be one of the following values:
+                    PENDING_VALIDATION
+                    SUCCESS
+                    FAILED
+        options(Dict[str, Any], optional): Currently, you can use this parameter to specify whether to add the certificate to a certificate
+            transparency log. Certificate transparency makes it possible to detect SSL/TLS certificates that
+            have been mistakenly or maliciously issued. Certificates that have not been logged typically
+            produce an error message in a browser. For more information, see Opting Out of Certificate
+            Transparency Logging. Defaults to None.
+            * CertificateTransparencyLoggingPreference (str, optional): You can opt out of certificate transparency
+                logging by specifying the DISABLED option. Opt in by specifying ENABLED.
         certificate_authority_arn(Text, optional): The Amazon Resource Name (ARN) of the private certificate authority
             (CA) that will be used to issue the certificate.,
-        tags(List, optional): One or more resource tags to associate with the certificate.
-            * Key (Text) --  The key of the tag.
-            * Value (Text) -- The value of the tag.
+        tags(Dict, optional): The collection of tags associated with the certificate. Defaults to None.
+            * (Key): The key of the tag.
+            * (Value, optional): The value of the tag.
         timeout(Dict, optional): Timeout configuration for request/import AWS Certificate.
             * create (Dict) -- Timeout configuration for request/imporng AWS Certificate
-                * delay -- The amount of time in seconds to wait between attempts.
-                * max_attempts -- Customized timeout configuration containing delay and max attempts.
+                * delay (int, Optional): The amount of time in seconds to wait between attempts.
+                * max_attempts (int, Optional): Customized timeout configuration containing delay and max attempts.
 
 
 
@@ -172,6 +250,15 @@ async def present(
     """
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     before = None
+    tags_list = None
+    tags_dict = None
+    if tags is not None:
+        if isinstance(tags, Dict):
+            tags_list = hub.tool.aws.tag_utils.convert_tag_dict_to_list(tags)
+            tags_dict = tags
+        else:
+            tags_list = tags
+            tags_dict = hub.tool.aws.tag_utils.convert_tag_list_to_dict(tags)
     if resource_id:
         before = await hub.exec.boto3.client.acm.describe_certificate(
             ctx, CertificateArn=resource_id
@@ -179,7 +266,6 @@ async def present(
 
     are_tags_updated = False
     is_reimported = False
-    old_tag_list = None
     if before and before["result"]:
         try:
             convert_ret = (
@@ -193,7 +279,9 @@ async def present(
             if not result["result"]:
                 result["comment"] = result["comment"] + convert_ret["comment"]
             result["old_state"] = convert_ret.get("ret")
-            old_tag_list = result["old_state"].get("tags")
+            old_tag_list = hub.tool.aws.tag_utils.convert_tag_dict_to_list(
+                result["old_state"].get("tags")
+            )
             plan_state = copy.deepcopy(result["old_state"])
 
             # Reimport certificate if certificate arn,private key and certificate body is given as input
@@ -224,16 +312,16 @@ async def present(
                         f"Would reimport certificate for aws.acm.certificate_manager {name}",
                     )
             if (
-                tags is not None
+                tags_list is not None
                 and not hub.tool.aws.state_comparison_utils.are_lists_identical(
-                    tags, old_tag_list
+                    tags_list, old_tag_list
                 )
             ):
                 update_tags_ret = await hub.exec.aws.acm.tag.update_tags(
                     ctx=ctx,
                     resource_id=resource_id,
                     old_tags=old_tag_list,
-                    new_tags=tags,
+                    new_tags=tags_list,
                 )
                 if not update_tags_ret["result"]:
                     result["comment"] = result["comment"] + update_tags_ret["comment"]
@@ -241,7 +329,7 @@ async def present(
                     return result
                 are_tags_updated = bool(update_tags_ret["ret"])
                 if ctx.get("test", False) and are_tags_updated:
-                    plan_state["tags"] = update_tags_ret["ret"].get("tags")
+                    plan_state["tags"] = update_tags_ret["ret"]
                     result["comment"] = result["comment"] + (
                         f"Would update tags for aws.acm.certificate_manager {name}",
                     )
@@ -271,7 +359,7 @@ async def present(
                             "certificate": certificate,
                             "private_key": private_key,
                             "certificate_chain": certificate_chain,
-                            "tags": tags,
+                            "tags": tags_dict,
                         },
                     )
                     result["comment"] = result["comment"] + (
@@ -283,7 +371,7 @@ async def present(
                     Certificate=certificate,
                     PrivateKey=private_key,
                     CertificateChain=certificate_chain,
-                    Tags=tags,
+                    Tags=tags_list,
                 )
                 result["result"] = imported["result"]
                 if not result["result"]:
@@ -339,7 +427,7 @@ async def present(
                             "domain_validation_options": domain_validation_options,
                             "options": options,
                             "certificate_authority_arn": certificate_authority_arn,
-                            "tags": tags,
+                            "tags": tags_dict,
                         },
                     )
                     result["comment"] = result[
@@ -358,7 +446,7 @@ async def present(
                     DomainValidationOptions=domain_validation_options,
                     Options=options,
                     CertificateAuthorityArn=certificate_authority_arn,
-                    Tags=tags,
+                    Tags=tags_list,
                 )
                 result["result"] = requested["result"]
                 if not result["result"]:
