@@ -5,12 +5,16 @@ hub.exec.boto3.client.cloudwatch.describe_alarms
 
 """
 import copy
+from dataclasses import field
+from dataclasses import make_dataclass
 from typing import Any
 from typing import Dict
 from typing import List
 
 __contracts__ = ["resource"]
 
+RESOURCE_TYPE = "aws.cloudwatch.metric_alarm"
+
 
 async def present(
     hub,
@@ -22,22 +26,74 @@ async def present(
     resource_id: str = None,
     alarm_description: str = None,
     actions_enabled: bool = True,
-    ok_actions: List = None,
-    alarm_actions: List = None,
-    insufficient_data_actions: List = None,
+    ok_actions: List[str] = None,
+    alarm_actions: List[str] = None,
+    insufficient_data_actions: List[str] = None,
     metric_name: str = None,
     namespace: str = None,
     statistic: str = None,
     extended_statistic: str = None,
-    dimensions: List = None,
+    dimensions: List[
+        make_dataclass("Dimension", [("Name", str), ("Value", str)])
+    ] = None,
     unit: str = None,
     datapoints_to_alarm: int = None,
     threshold: float = None,
     treat_missing_data: str = None,
     evaluate_low_sample_count_percentile: str = None,
-    metrics: List = None,
+    metrics: List[
+        make_dataclass(
+            "MetricDataQuery",
+            [
+                ("Id", str),
+                (
+                    "MetricStat",
+                    make_dataclass(
+                        "MetricStat",
+                        [
+                            (
+                                "Metric",
+                                make_dataclass(
+                                    "Metric",
+                                    [
+                                        ("Namespace", str, field(default=None)),
+                                        ("MetricName", str, field(default=None)),
+                                        (
+                                            "Dimensions",
+                                            List[
+                                                make_dataclass(
+                                                    "Dimension",
+                                                    [("Name", str), ("Value", str)],
+                                                )
+                                            ],
+                                            field(default=None),
+                                        ),
+                                    ],
+                                ),
+                            ),
+                            ("Period", int),
+                            ("Stat", str),
+                            ("Unit", str, field(default=None)),
+                        ],
+                    ),
+                    field(default=None),
+                ),
+                ("Expression", str, field(default=None)),
+                ("Label", str, field(default=None)),
+                ("ReturnData", bool, field(default=None)),
+                ("Period", int, field(default=None)),
+                ("AccountId", str, field(default=None)),
+            ],
+        )
+    ] = None,
     threshold_metric_id: str = None,
-    tags: List = None,
+    tags: Dict[str, Any]
+    or List[
+        make_dataclass(
+            "Tag",
+            [("Key", str, field(default=None)), ("Value", str, field(default=None))],
+        )
+    ] = None,
 ):
     r"""
 
@@ -59,10 +115,8 @@ async def present(
         resource_id(Text): The AWS name of the metric alarm.
         alarm_description(Text, Optional): The description for the alarm.
         actions_enabled(Boolean, Optional): Indicates whether actions should be executed during any changes to the alarm
-        state.
-            The default is TRUE .
-        ok_actions(List, Optional): The actions to execute when this alarm transitions to an OK state from any other
-        state.
+            state. The default is TRUE .
+        ok_actions(List, Optional): The actions to execute when this alarm transitions to an OK state from any other state.
             Each action is specified as an Amazon Resource Name (ARN).
             Valid Values:
                 arn:aws:automate:*region* :ec2:stop | arn:aws:automate:*region* :ec2:terminate |
@@ -76,7 +130,7 @@ async def present(
                 arn:aws:swf:*region* :*account-id* :action/actions/AWS_EC2.InstanceId.Reboot/1.0 |
                 arn:aws:swf:*region* :*account-id* :action/actions/AWS_EC2.InstanceId.Recover/1.0
         alarm_actions(List, Optional):The actions to execute when this alarm transitions to the ALARM state from any
-        other state.
+            other state.
             Each action is specified as an Amazon Resource Name (ARN).
             Valid Values:
                 arn:aws:automate:*region* :ec2:stop |
@@ -117,14 +171,11 @@ async def present(
         extended_statistic(Text, Optional): The percentile statistic for the metric specified in MetricName . Specify a value
             between p0.0 and p100. When you call PutMetricAlarm and specify a MetricName , you must specify either
             Statistic or ExtendedStatistic, but not both.
-        dimensions(List, Optional): The dimensions for the metric specified in MetricName. A dimension is a name/value pair that
-            is part of the identity of a metric. You can assign up to 10 dimensions to a metric. Because dimensions are
-            part of the unique identifier for a metric, whenever you add a unique name/value pair to one of your
-            metrics, you are creating a new variation of that metric.
-            * Name (Text) -- The name of the dimension. Dimension names must contain only ASCII characters and must
-                include at least one non-whitespace character.
-            * Value (Text) -- The value of the dimension. Dimension values must contain only ASCII characters and must
-                include at least one non-whitespace character.
+        dimensions(List[Dict[str, Any]], optional): The dimensions for the metric specified in MetricName. Defaults to None.
+            * Name (str): The name of the dimension. Dimension names must contain only ASCII characters, must include at
+                least one non-whitespace character, and cannot start with a colon (:).
+            * Value (str): The value of the dimension. Dimension values must contain only ASCII characters and must include
+                at least one non-whitespace character.
         period(Integer): The length, in seconds, used each time the metric specified in MetricName is evaluated.
             Valid values are 10, 30, and any multiple of 60.
         unit(Text, Optional): The unit of measure for the statistic. For example, the units for the Amazon EC2 NetworkIn metric are
@@ -152,16 +203,83 @@ async def present(
             the alarm state does not change during periods with too few data points to be statistically significant.
             If you specify evaluate or omit this parameter, the alarm is always evaluated and possibly changes state
             no matter how many data points are available.
-        metrics(List, Optional): An array of MetricDataQuery structures that enable you to create an alarm based on the result
-            of a metric math expression. For each PutMetricAlarm operation, you must specify either MetricName or a
-            Metrics array.
-        threshold_metric_id(Text, Optional): If this is an alarm based on an anomaly detection model, make this value match the
-            ID of the ANOMALY_DETECTION_BAND function.
-        tags(List, Optional): A list of key-value pairs to associate with the alarm. You can associate as many as 50 tags with an
-            alarm.
-            * Key (Text) -- A string that you can use to assign a value. The combination of tag keys and values can
-                help you organize and categorize your resources.
-            * Value (Text) -- The value for the specified tag key.
+        metrics(List[Dict[str, Any]], optional): An array of MetricDataQuery structures that enable you to create an alarm based on the result of
+            a metric math expression. For each PutMetricAlarm operation, you must specify either MetricName
+            or a Metrics array. Each item in the Metrics array either retrieves a metric or performs a math
+            expression. One item in the Metrics array is the expression that the alarm watches. You
+            designate this expression by setting ReturnData to true for this object in the array. For more
+            information, see MetricDataQuery. If you use the Metrics parameter, you cannot include the
+            MetricName, Dimensions, Period, Namespace, Statistic, or ExtendedStatistic parameters of
+            PutMetricAlarm in the same operation. Instead, you retrieve the metrics you are using in your
+            math expression as part of the Metrics array. Defaults to None.
+            * Id (str): A short name used to tie this object to the results in the response. This name must be unique
+                within a single call to GetMetricData. If you are performing math expressions on this set of
+                data, this name represents that data and can serve as a variable in the mathematical expression.
+                The valid characters are letters, numbers, and underscore. The first character must be a
+                lowercase letter.
+            * MetricStat (Dict[str, Any], optional): The metric to be returned, along with statistics, period, and units. Use this parameter only if
+                this object is retrieving a metric and not performing a math expression on returned data. Within
+                one MetricDataQuery object, you must specify either Expression or MetricStat but not both.
+                * Metric (Dict[str, Any]): The metric to return, including the metric name, namespace, and dimensions.
+                    * Namespace (str, optional): The namespace of the metric.
+                    * MetricName (str, optional): The name of the metric. This is a required field.
+                    * Dimensions (List[Dict[str, Any]], optional): The dimensions for the metric.
+                        * Name (str): The name of the dimension. Dimension names must contain only ASCII characters, must include at
+                            least one non-whitespace character, and cannot start with a colon (:).
+                        * Value (str): The value of the dimension. Dimension values must contain only ASCII characters and must include
+                            at least one non-whitespace character.
+                * Period (int): The granularity, in seconds, of the returned data points. For metrics with regular resolution, a
+                    period can be as short as one minute (60 seconds) and must be a multiple of 60. For high-
+                    resolution metrics that are collected at intervals of less than one minute, the period can be 1,
+                    5, 10, 30, 60, or any multiple of 60. High-resolution metrics are those metrics stored by a
+                    PutMetricData call that includes a StorageResolution of 1 second. If the StartTime parameter
+                    specifies a time stamp that is greater than 3 hours ago, you must specify the period as follows
+                    or no data points in that time range is returned:   Start time between 3 hours and 15 days ago -
+                    Use a multiple of 60 seconds (1 minute).   Start time between 15 and 63 days ago - Use a
+                    multiple of 300 seconds (5 minutes).   Start time greater than 63 days ago - Use a multiple of
+                    3600 seconds (1 hour).
+                * Stat (str): The statistic to return. It can include any CloudWatch statistic or extended statistic.
+                * Unit (str, optional): When you are using a Put operation, this defines what unit you want to use when storing the
+                    metric. In a Get operation, if you omit Unit then all data that was collected with any unit is
+                    returned, along with the corresponding units that were specified when the data was reported to
+                    CloudWatch. If you specify a unit, the operation returns only data that was collected with that
+                    unit specified. If you specify a unit that does not match the data collected, the results of the
+                    operation are null. CloudWatch does not perform unit conversions.
+            * Expression (str, optional): This field can contain either a Metrics Insights query, or a metric math expression to be
+                performed on the returned data. For more information about Metrics Insights queries, see Metrics
+                Insights query components and syntax in the Amazon CloudWatch User Guide. A math expression can
+                use the Id of the other metrics or queries to refer to those metrics, and can also use the Id of
+                other expressions to use the result of those expressions. For more information about metric math
+                expressions, see Metric Math Syntax and Functions in the Amazon CloudWatch User Guide. Within
+                each MetricDataQuery object, you must specify either Expression or MetricStat but not both.
+            * Label (str, optional): A human-readable label for this metric or expression. This is especially useful if this is an
+                expression, so that you know what the value represents. If the metric or expression is shown in
+                a CloudWatch dashboard widget, the label is shown. If Label is omitted, CloudWatch generates a
+                default. You can put dynamic expressions into a label, so that it is more descriptive. For more
+                information, see Using Dynamic Labels.
+            * ReturnData (bool, optional): When used in GetMetricData, this option indicates whether to return the timestamps and raw data
+                values of this metric. If you are performing this call just to do math expressions and do not
+                also need the raw data returned, you can specify False. If you omit this, the default of True is
+                used. When used in PutMetricAlarm, specify True for the one expression result to use as the
+                alarm. For all other metrics and expressions in the same PutMetricAlarm operation, specify
+                ReturnData as False.
+            * Period (int, optional): The granularity, in seconds, of the returned data points. For metrics with regular resolution, a
+                period can be as short as one minute (60 seconds) and must be a multiple of 60. For high-
+                resolution metrics that are collected at intervals of less than one minute, the period can be 1,
+                5, 10, 30, 60, or any multiple of 60. High-resolution metrics are those metrics stored by a
+                PutMetricData operation that includes a StorageResolution of 1 second.
+            * AccountId (str, optional): The ID of the account where the metrics are located, if this is a cross-account alarm. Use this
+                field only for PutMetricAlarm operations. It is not used in GetMetricData operations.
+        tags(List or Dict, Optional): A List of tags in the format of [{"Key": tag-key, "Value": tag-value}] or dict in the format of
+            {tag-key: tag-value} to associate with the alarm. You can associate as many as 50 tags with
+            an alarm. Tags can help you organize and categorize your resources. You can also use them to
+            scope user permissions by granting a user permission to access or change only resources with
+            certain tag values. If you are using this operation to update an existing alarm, any tags you
+            specify in this parameter are ignored. To change the tags of an existing alarm, use TagResource
+            or UntagResource. Defaults to None.
+            * (Key): A string that you can use to assign a value. The combination of tag keys and values can help you
+                organize and categorize your resources.
+            * (Value): The value for the specified tag key.
 
     Request Syntax:
         [metric-alarm-name]::
@@ -226,6 +344,10 @@ async def present(
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     resource_updated = False
     before = None
+
+    if isinstance(tags, List):
+        tags = hub.tool.aws.tag_utils.convert_tag_list_to_dict(tags)
+
     resource_parameters = {
         "AlarmName": name,
         "AlarmDescription": alarm_description,
@@ -248,7 +370,7 @@ async def present(
         "EvaluateLowSampleCountPercentile": evaluate_low_sample_count_percentile,
         "Metrics": metrics,
         "ThresholdMetricId": threshold_metric_id,
-        "Tags": tags,
+        "Tags": hub.tool.aws.tag_utils.convert_tag_dict_to_list(tags),
     }
 
     if resource_id:
@@ -257,8 +379,10 @@ async def present(
         )
         before = await hub.tool.boto3.resource.describe(resource)
 
-    if before:
-        try:
+    try:
+
+        if before:
+
             convert_ret = await hub.tool.aws.cloudwatch.conversion_utils.convert_raw_metric_alarm_to_present(
                 ctx, raw_resource=before, idem_resource_name=name
             )
@@ -296,35 +420,28 @@ async def present(
                 ]:
                     if key in update_ret["ret"]:
                         plan_state[key] = update_ret["ret"][key]
-            if (tags is not None) and (
-                not hub.tool.aws.state_comparison_utils.are_lists_identical(
-                    tags, result["old_state"].get("tags")
-                )
-            ):
+            if tags is not None and tags != result["old_state"].get("tags"):
                 # Update tags
                 update_tag_ret = (
                     await hub.exec.aws.cloudwatch.metric_alarm.update_metric_tags(
                         ctx=ctx,
                         alarm_arn=before.get("AlarmArn"),
-                        old_tags=result["old_state"].get("tags", []),
+                        old_tags=result["old_state"].get("tags", {}),
                         new_tags=tags,
                     )
                 )
                 result["result"] = result["result"] and update_tag_ret["result"]
                 result["comment"] = result["comment"] + update_tag_ret["comment"]
-                resource_updated = resource_updated or bool(update_tag_ret["ret"])
+                resource_updated = resource_updated or bool(update_tag_ret["result"])
+
                 if ctx.get("test", False) and update_tag_ret["ret"] is not None:
-                    plan_state["tags"] = update_tag_ret["ret"].get("tags")
+                    plan_state["tags"] = update_tag_ret["ret"]
 
             if not resource_updated:
                 result["comment"] = result["comment"] + (
                     f"aws.cloudwatch.metric_alarm '{name}' has no property need to be updated.",
                 )
-        except hub.tool.boto3.exception.ClientError as e:
-            result["comment"] = result["comment"] + (f"{e.__class__.__name__}: {e}",)
-            result["result"] = False
-    else:
-        try:
+        else:
             if ctx.get("test", False):
                 result["new_state"] = hub.tool.aws.test_state_utils.generate_test_state(
                     enforced_state={},
@@ -353,8 +470,8 @@ async def present(
                         "threshold_metric_id": threshold_metric_id,
                     },
                 )
-                result["comment"] = (
-                    f"Would create aws.cloudwatch.metric_alarm {name}",
+                result["comment"] = hub.tool.aws.comment_utils.would_create_comment(
+                    resource_type=RESOURCE_TYPE, name=name
                 )
                 return result
 
@@ -366,12 +483,10 @@ async def present(
             if not result["result"]:
                 result["comment"] = result["comment"] + ret["comment"]
                 return result
-            result["comment"] = result["comment"] + (f"Created '{name}'",)
-        except hub.tool.boto3.exception.ClientError as e:
-            result["comment"] = result["comment"] + (f"{e.__class__.__name__}: {e}",)
-            result["result"] = False
+            result["comment"] = result[
+                "comment"
+            ] + hub.tool.aws.comment_utils.create_comment(RESOURCE_TYPE, name)
 
-    try:
         if ctx.get("test", False):
             result["new_state"] = plan_state
         elif (not before) or resource_updated:
@@ -420,7 +535,7 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     if not resource_id:
         result["comment"] = hub.tool.aws.comment_utils.already_absent_comment(
-            resource_type="aws.cloudwatch.metric_alarm", name=name
+            resource_type=RESOURCE_TYPE, name=name
         )
         return result
     resource = hub.tool.boto3.resource.create(ctx, "cloudwatch", "Alarm", resource_id)
@@ -428,7 +543,7 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
 
     if not before:
         result["comment"] = hub.tool.aws.comment_utils.already_absent_comment(
-            resource_type="aws.cloudwatch.metric_alarm", name=name
+            resource_type=RESOURCE_TYPE, name=name
         )
     elif ctx.get("test", False):
         convert_ret = await hub.tool.aws.cloudwatch.conversion_utils.convert_raw_metric_alarm_to_present(
@@ -439,7 +554,7 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
             result["comment"] = result["comment"] + convert_ret["comment"]
         result["old_state"] = convert_ret["ret"]
         result["comment"] = hub.tool.aws.comment_utils.would_delete_comment(
-            resource_type="aws.cloudwatch.metric_alarm", name=name
+            resource_type=RESOURCE_TYPE, name=name
         )
         return result
     else:
@@ -460,7 +575,7 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
                 result["result"] = False
                 return result
             result["comment"] = hub.tool.aws.comment_utils.delete_comment(
-                resource_type="aws.cloudwatch.metric_alarm", name=name
+                resource_type=RESOURCE_TYPE, name=name
             )
 
         except hub.tool.boto3.exception.ClientError as e:
