@@ -1,4 +1,7 @@
+"""State module for managing EC2 AMIs."""
 import copy
+from dataclasses import field
+from dataclasses import make_dataclass
 from datetime import datetime
 from typing import Any
 from typing import Dict
@@ -23,32 +26,36 @@ async def present(
     sriov_net_support: str = None,
     virtualization_type: str = None,
     boot_mode: str = None,
-    tags: List = None,
+    tags: Dict[str, Any]
+    or List[
+        make_dataclass(
+            "Tag",
+            [("Key", str, field(default=None)), ("Value", str, field(default=None))],
+        )
+    ] = None,
     resource_id: str = None,
 ) -> Dict[str, Any]:
-    r"""
-
-    Register an AWS AMI.
+    """Register an AWS AMI.
 
     Args:
-        name(Text):
+        name(str):
             A region-unique name for the AMI.
-            Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.),
-            slashes (/), dashes (-), single quotes ('), at-signs (@), or underscores(_)
+            Constraints: 3-128 alphanumeric characters, parentheses (``()``), square brackets (``[]``), spaces, periods (``.``),
+            slashes (``/``), dashes (``-``), single quotes (``'``), at-signs (``@``), or underscores(``_``)
 
-        resource_id(Text, optional):
+        resource_id(str, optional):
             AWS AMI image_id
 
-        image_location(Text, optional):
+        image_location(str, optional):
             The full path to your AMI manifest in Amazon S3 storage.
             The specified bucket must have the aws-exec-read canned access control list (ACL) to ensure that it can be accessed by Amazon EC2.
             For more information, see Canned ACLs in the Amazon S3 Service Developer Guide .
 
-        architecture(Text, optional):
+        architecture(str, optional):
             The architecture of the AMI. Default: For Amazon EBS-backed AMIs, i386.
             For instance store-backed AMIs, the architecture specified in the manifest file.
 
-        block_device_mappings(List, optional):
+        block_device_mappings(list, optional):
             The block device mapping entries.
             If you specify an Amazon EBS volume using the ID of an Amazon EBS snapshot, you can't specify the encryption state of the volume.
             If you create an AMI on an Outpost, then all backing snapshots must be on the same Outpost or in the Region of that Outpost.
@@ -56,196 +63,213 @@ async def present(
             For more information, Amazon EBS local snapshots on Outposts in the Amazon Elastic Compute Cloud User Guide.
 
             Below are the attributes available for block device mapping entries:
+
             Type: object dict(Describes a block device mapping, which defines the EBS volumes and instance store volumes to attach to an instance at launch.)
 
-            DeviceName (Text):
-            The device name (for example, /dev/sdh or xvdh ).
+            * DeviceName (str):
+                The device name (for example, ``/dev/sdh`` or ``xvdh``).
 
-            VirtualName (Text, Optional):
-            The virtual device name (ephemeral N). Instance store volumes are numbered starting from 0.
-            An instance type with 2 available instance store volumes can specify mappings for ephemeral0 and ephemeral1 .
-            The number of available instance store volumes depends on the instance type. After you connect to the instance, you must mount the volume.
-            NVMe instance store volumes are automatically enumerated and assigned a device name. Including them in your block device mapping has no effect.
-            Constraints: For M3 instances, you must specify instance store volumes in the block device mapping for the instance.
-            When you launch an M3 instance, we ignore any instance store volumes specified in the block device mapping for the AMI.
+            * VirtualName (str, Optional):
+                The virtual device name (ephemeral N). Instance store volumes are numbered starting from 0.
+                An instance type with 2 available instance store volumes can specify mappings for ephemeral0 and ephemeral1 .
+                The number of available instance store volumes depends on the instance type. After you connect to the instance, you must mount the volume.
+                NVMe instance store volumes are automatically enumerated and assigned a device name. Including them in your block device mapping has no effect.
+                Constraints: For M3 instances, you must specify instance store volumes in the block device mapping for the instance.
+                When you launch an M3 instance, we ignore any instance store volumes specified in the block device mapping for the AMI.
 
-            Ebs (Dict):
-            Parameters used to automatically set up EBS volumes when the instance is launched.
+            * Ebs (dict):
+                Parameters used to automatically set up EBS volumes when the instance is launched.
 
-            DeleteOnTermination (bool, Optional):
-            Indicates whether the EBS volume is deleted on instance termination. For more information,
-            see Preserving Amazon EBS volumes on instance termination in the Amazon EC2 User Guide .
+            * DeleteOnTermination (bool, optional):
+                Indicates whether the EBS volume is deleted on instance termination. For more information,
+                see Preserving Amazon EBS volumes on instance termination in the Amazon EC2 User Guide .
 
-            Iops (Integer, Optional):
-            The number of I/O operations per second (IOPS). For gp3 , io1 , and io2 volumes,
-            this represents the number of IOPS that are provisioned for the volume. For gp2 volumes,
-            this represents the baseline performance of the volume and the rate at which the volume accumulates I/O credits for bursting.
+            * Iops (int, optional):
+                The number of I/O operations per second (IOPS). For gp3 , io1 , and io2 volumes,
+                this represents the number of IOPS that are provisioned for the volume. For gp2 volumes,
+                this represents the baseline performance of the volume and the rate at which the volume accumulates I/O credits for bursting.
 
-            The following are the supported values for each volume type:
+                The following are the supported values for each volume type:
 
-            gp3 : 3,000-16,000 IOPS
-            io1 : 100-64,000 IOPS
-            io2 : 100-64,000 IOPS
-            For io1 and io2 volumes, we guarantee 64,000 IOPS only for Instances built on the Nitro System.
-            Other instance families guarantee performance up to 32,000 IOPS.
+                gp3 : 3,000-16,000 IOPS
+                io1 : 100-64,000 IOPS
+                io2 : 100-64,000 IOPS
+                For io1 and io2 volumes, we guarantee 64,000 IOPS only for Instances built on the Nitro System.
+                Other instance families guarantee performance up to 32,000 IOPS.
 
-            This parameter is required for io1 and io2 volumes. The default for gp3 volumes is 3,000 IOPS.
-            This parameter is not supported for gp2 , st1 , sc1 , or standard volumes.
+                This parameter is required for io1 and io2 volumes. The default for gp3 volumes is 3,000 IOPS.
+                This parameter is not supported for gp2 , st1 , sc1 , or standard volumes.
 
-            SnapshotId (Text):
-            The ID of the snapshot.
+            * SnapshotId (str):
+                The ID of the snapshot.
 
-            VolumeSize (Integer):
-            The size of the volume, in GiBs. You must specify either a snapshot ID or a volume size.
-            If you specify a snapshot, the default is the snapshot size.
-            You can specify a volume size that is equal to or larger than the snapshot size.
+            * VolumeSize (int):
+                The size of the volume, in GiBs. You must specify either a snapshot ID or a volume size.
+                If you specify a snapshot, the default is the snapshot size.
+                You can specify a volume size that is equal to or larger than the snapshot size.
 
-            The following are the supported volumes sizes for each volume type:
+                The following are the supported volumes sizes for each volume type:
 
-            gp2 and gp3 :1-16,384
-            io1 and io2 : 4-16,384
-            st1 and sc1 : 125-16,384
-            standard : 1-1,024
+                gp2 and gp3 :1-16,384
+                io1 and io2 : 4-16,384
+                st1 and sc1 : 125-16,384
+                standard : 1-1,024
 
-            VolumeType (Text):
-            The volume type. For more information, see Amazon EBS volume types in the Amazon EC2 User Guide .
-            If the volume type is io1 or io2 , you must specify the IOPS that the volume supports.
+            * VolumeType (str):
+                The volume type. For more information, see Amazon EBS volume types in the Amazon EC2 User Guide .
+                If the volume type is io1 or io2 , you must specify the IOPS that the volume supports.
 
-            KmsKeyId (Text, Optional):
-            Identifier (key ID, key alias, ID ARN, or alias ARN) for a customer managed CMK under which the EBS volume is encrypted.
-            This parameter is only supported on BlockDeviceMapping objects called by RunInstances, RequestSpotFleet, and RequestSpotInstances .
+            * KmsKeyId (str, optional):
+                Identifier (key ID, key alias, ID ARN, or alias ARN) for a customer managed CMK under which the EBS volume is encrypted.
+                This parameter is only supported on BlockDeviceMapping objects called by RunInstances, RequestSpotFleet, and RequestSpotInstances .
 
-            Throughput (Integer, Optional):
-            The throughput that the volume supports, in MiB/s.
-            This parameter is valid only for gp3 volumes.
-            Valid Range: Minimum value of 125. Maximum value of 1000.
+            * Throughput (int, optional):
+                The throughput that the volume supports, in MiB/s.
+                This parameter is valid only for gp3 volumes.
+                Valid Range: Minimum value of 125. Maximum value of 1000.
 
-            OutpostArn (Text, Optional):
-            The ARN of the Outpost on which the snapshot is stored.
+            * OutpostArn (str, optional):
+                The ARN of the Outpost on which the snapshot is stored.
 
-            Encrypted (bool, Optional):
-            Indicates whether the encryption state of an EBS volume is changed while being restored from a backing snapshot.
-            The effect of setting the encryption state to true depends on the volume origin (new or from a snapshot),
-            starting encryption state, ownership, and whether encryption by default is enabled. For more information,
-            see Amazon EBS encryption in the Amazon EC2 User Guide .
-            In no case can you remove encryption from an encrypted volume.
-            Encrypted volumes can only be attached to instances that support Amazon EBS encryption.
-            For more information, see Supported instance types .
-            This parameter is not returned by DescribeImageAttribute .
+            * Encrypted (bool, optional):
+                Indicates whether the encryption state of an EBS volume is changed while being restored from a backing snapshot.
+                The effect of setting the encryption state to true depends on the volume origin (new or from a snapshot),
+                starting encryption state, ownership, and whether encryption by default is enabled. For more information,
+                see Amazon EBS encryption in the Amazon EC2 User Guide .
+                In no case can you remove encryption from an encrypted volume.
+                Encrypted volumes can only be attached to instances that support Amazon EBS encryption.
+                For more information, see Supported instance types .
+                This parameter is not returned by DescribeImageAttribute .
 
-            NoDevice (Text, Optional):
-            To omit the device from the block device mapping, specify an empty string.
-            When this property is specified, the device is removed from the block device mapping regardless of the assigned value.
+            * NoDevice (str, optional):
+                To omit the device from the block device mapping, specify an empty string.
+                When this property is specified, the device is removed from the block device mapping regardless of the assigned value.
 
-        description(Text, optional):
+        description(str, optional):
             A description for your AMI.
 
-        ena_support(Boolean, optional):
+        ena_support(bool, optional):
             Set to true to enable enhanced networking with ENA for the AMI and any instances that you launch from the AMI.
             This option is supported only for HVM AMIs.
             Specifying this option with a PV AMI can make instances launched from the AMI unreachable.
 
-        kernel_id (Text, optional):
+        kernel_id (str, optional):
             The ID of the kernel.
 
-        billing_products (List, optional):
+        billing_products (list, optional):
             The billing product codes. Your account must be authorized to specify billing product codes.
             Otherwise, you can use the Amazon Web Services Marketplace to bill for the use of an AMI.
 
-        ramdisk_id (Text, optional):
+        ramdisk_id(str, optional):
             The ID of the RAM disk.
 
-        root_device_name (Text, optional):
+        root_device_name(str, optional):
             The device name of the root device volume (for example, /dev/sda1 ).
 
-        sriov_net_support (Text, optional):
+        sriov_net_support(str, optional):
             Set to simple to enable enhanced networking with the Intel 82599 Virtual Function interface for the AMI and any instances that you launch from the AMI.
             There is no way to disable sriovNetSupport at this time.
             This option is supported only for HVM AMIs. Specifying this option with a PV AMI can make instances launched from the AMI unreachable.
 
-        virtualization_type (Text, optional):
+        virtualization_type(str, optional):
             The type of virtualization (hvm | paravirtual).
             Default: paravirtual
 
-        boot_mode (Text, optional):
+        boot_mode(str, optional):
             The boot mode of the AMI. For more information, see Boot modes in the Amazon Elastic Compute Cloud User Guide.
 
-        tags(List, optional):
-            The tags to assign to the AMI. Defaults to None.
-                * Key (Text) -- The key of the tag. Tag keys are case-sensitive and accept a maximum of 127 Unicode characters. May not begin with aws: .
-                * Value (Text) -- The value of the tag. Tag values are case-sensitive and accept a maximum of 255 Unicode characters.
+        tags(dict or list, optional):
+            Dict in the format of ``{tag-key: tag-value}`` or List of tags in the format of
+            ``[{"Key": tag-key, "Value": tag-value}]`` to associate with the AMI.
+
+            * Key (str):
+                The key name that can be used to look up or retrieve the associated value. For example,
+                Department or Cost Center are common choices.
+
+            * Value (str):
+                The value associated with this tag. For example, tags with a key name of Department could have
+                values such as Human Resources, Accounting, and Support. Tags with a key name of Cost Center
+                might have values that consist of the number associated with the different cost centers in your
+                company. Typically, many resources have tags with the same key name but with different values.
+                Amazon Web Services always interprets the tag Value as a single string. If you need to store an
+                array, you can store comma-separated values in the string. However, you must interpret the value
+                in your code.
 
     Request Syntax:
-       [ami-resource-id]:
-         aws.ec2.ami.present:
-            - name: 'string'
-            - image_location: 'string'
-            - architecture: 'i386'|'x86_64'|'arm64'|'x86_64_mac'
-            - block_device_mappings:
-            - DeviceName: 'string'
-                VirtualName: 'string'
+       .. code-block:: sls
+
+          [ami-resource-id]:
+            aws.ec2.ami.present:
+              - name: "string"
+              - image_location: "string"
+              - architecture: i386|x86_64|arm64|x86_64_mac
+              - block_device_mappings:
+                  - DeviceName: "string"
+                    VirtualName: "string"
                     Ebs:
                       DeleteOnTermination: True|False
                       Iops: 123
-                      SnapshotId: 'string'
+                      SnapshotId: "string"
                       VolumeSize: integer
-                      VolumeType': 'standard'|'io1'|'io2'|'gp2'|'sc1'|'st1'|'gp3'
-                      KmsKeyId: 'string'
+                      VolumeType: standard|io1|io2|gp2|sc1|st1|gp3
+                      KmsKeyId: "string"
                       Throughput: integer
-                      OutpostArn: 'string'
+                      OutpostArn: "string"
                       Encrypted: True|False
-                NoDevice: 'string'
-            - description: 'string'
-            - ena_support: True|False
-            - kernel_id: 'string'
-            - billing_products
-                - 'string'
-                - 'string'
-            - ramdisk_id: 'string'
-            - root_device_name: 'string'
-            - sriov_net_support: 'string'
-            - virtualization_type: 'string'
-            - boot_mode: 'legacy-bios'|'uefi'
-            - tags:
-                - Key: 'string'
-                  Value: 'string'
-            - resource_id: 'string'
+                      NoDevice: "string"
+              - description: "string"
+              - ena_support: True|False
+              - kernel_id: "string"
+              - billing_products:
+                  - string
+                  - string
+              - ramdisk_id: "string"
+              - root_device_name: "string"
+              - sriov_net_support: "string"
+              - virtualization_type: "string"
+              - boot_mode: legacy-bios|uefi
+              - tags:
+                  - Key: "string"
+                    Value: "string"
+              - resource_id: "string"
 
     Returns:
         Dict[str, Any]
 
     Examples:
-
         .. code-block:: sls
 
             idem-test-ami:
               aws.ec2.ami.present:
-              - name: amzn-ami-vpc-nat-2018.03.0.20210721.0-x86_64-ebs
-              - resource_id: ami-00a36856283d67c39
-              - image_location: amazon/amzn-ami-vpc-nat-2018.03.0.20210721.0-x86_64-ebs
-              - architecture: x86_64
-              - kernel_id: None
-              - block_device_mappings:
-                - DeviceName: /dev/xvda
-                  Ebs:
-                    DeleteOnTermination: false
-                    SnapshotId: snap-1833745e
-                    VolumeSize: 15
-                    VolumeType: standard
-              - description: Amazon Linux AMI 2018.03.0.20210721.0 x86_64 VPC HVM ebs
-              - ramdisk_id: ari-1a2b3c4d
-              - root_device_name: /dev/xvda
-              - virtualization_type: hvm
-              - tags:
-                - Key: Name
-                  Value: ami-name
-                - Key: ami-tag-key-2
-                  Value: ami-tag-value-2
+                - name: amzn-ami-vpc-nat-2018.03.0.20210721.0-x86_64-ebs
+                - resource_id: ami-00a36856283d67c39
+                - image_location: amazon/amzn-ami-vpc-nat-2018.03.0.20210721.0-x86_64-ebs
+                - architecture: x86_64
+                - kernel_id: None
+                - block_device_mappings:
+                    - DeviceName: /dev/xvda
+                      Ebs:
+                        DeleteOnTermination: false
+                        SnapshotId: snap-1833745e
+                        VolumeSize: 15
+                        VolumeType: standard
+                - description: Amazon Linux AMI 2018.03.0.20210721.0 x86_64 VPC HVM ebs
+                - ramdisk_id: ari-1a2b3c4d
+                - root_device_name: /dev/xvda
+                - virtualization_type: hvm
+                - tags:
+                    - Key: Name
+                      Value: ami-name
+                    - Key: ami-tag-key-2
+                      Value: ami-tag-value-2
     """
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     before = None
     resource_updated = False
 
+    if isinstance(tags, List):
+        tags = hub.tool.aws.tag_utils.convert_tag_list_to_dict(tags)
+
     desired_state = {
         "name": name,
         "image_location": image_location,
@@ -300,7 +324,7 @@ async def present(
         plan_state = copy.deepcopy(result["old_state"])
 
         # update (we can only update tags and description)
-        if tags is not None:
+        if tags is not None and tags != result["old_state"].get("tags"):
             # Update tags
             update_ret = await hub.exec.aws.ec2.tag.update_tags(
                 ctx=ctx,
@@ -314,9 +338,9 @@ async def present(
             # If tag updates fails, it wont update description
             if not result["result"]:
                 return result
-            resource_updated = bool(update_ret["ret"])
-            if ctx.get("test", False) and update_ret["ret"]:
-                plan_state["tags"] = update_ret["ret"].get("tags")
+            resource_updated = resource_updated or update_ret["result"]
+            if ctx.get("test", False) and update_ret["result"]:
+                plan_state["tags"] = update_ret["ret"]
 
         if (
             description is not None
@@ -373,7 +397,9 @@ async def present(
 
         if tags is not None:
             ret = await hub.exec.boto3.client.ec2.create_tags(
-                ctx, Resources=[resource_id], Tags=tags
+                ctx,
+                Resources=[resource_id],
+                Tags=hub.tool.aws.tag_utils.convert_tag_dict_to_list(tags),
             )
             result["result"] = ret["result"]
             if not result["result"]:
@@ -402,9 +428,9 @@ async def present(
 
 
 async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]:
-    r"""
+    """Deregisters the specified AMI.
 
-    Deregisters the specified AMI. After you deregister an AMI, it can't be used to launch new instances.
+    After you deregister an AMI, it can't be used to launch new instances.
 
     If you deregister an AMI that matches a Recycle Bin retention rule,
     the AMI is retained in the Recycle Bin for the specified retention period.
@@ -419,26 +445,27 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
     it doesn't affect the files that you uploaded to Amazon S3 when you created the AMI.
 
     Args:
-        name(Text): The Idem name of the AMI.
-        resource_id(Text, optional): The AWS image_id.
+        name(str): The Idem name of the AMI.
+        resource_id(str, optional): The AWS image_id.
 
     Request Syntax:
-       [ami-resource-id]:
+        .. code-block:: sls
+
+            [ami-resource-id]:
               aws.ec2.ami.absent:
-              - name: 'string'
-              - resource_id: 'string'
+                - name: "string"
+                - resource_id: "string"
 
     Returns:
         Dict[str, Any]
 
     Examples:
-
         .. code-block:: sls
 
             idem-test-ami:
               aws.ec2.ami.absent:
                 - name: idem-test-ami
-                - resource_id:ami-000c540e28953ace2
+                - resource_id: ami-000c540e28953ace2
     """
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     if not resource_id:
@@ -479,7 +506,7 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
 
 async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
     r"""
-    Describe the resource in a way that can be recreated/managed with the corresponding "present" function
+    Describe the resource in a way that can be recreated/managed with the corresponding "present" function.
 
     Gets information about the AWS AMI
 
@@ -487,7 +514,6 @@ async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
         Dict[str, Dict[str, Any]]
 
     Examples:
-
         .. code-block:: bash
 
             $ idem describe aws.ec2.ami
@@ -531,10 +557,10 @@ async def search(
     Use an un-managed AMI as a data-source. Supply one of the inputs as the filter.
 
     Args:
-        name(Text):
+        name(str):
             The name of the Idem state.
 
-        resource_id(Text, optional):
+        resource_id(str, optional):
             AWS image id to identify the resource.
 
         executable_users(list, optional):
@@ -555,33 +581,36 @@ async def search(
 
         filters(list, optional):
             One or more filters: for example, tag :<key>, tag-key. A complete list of filters can be found at
-            https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_images
+            `boto3: EC2.Client.describe_images <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_images>`__
 
 
     Request Syntax:
-        [Idem-state-name]:
-          aws.ec2.ami.search:
-          - resource_id: 'string'
-          - executable_users
-            - user1
-          - owners
-            - amazon
-          - include_deprecated: True|False
-          - most_recent: True|False
-          - filters:
-            - name: 'string'
-              values: 'list'
-            - name: 'string'
-              values: 'list'
+        .. code-block:: sls
 
-        Examples:
+            [Idem-state-name]:
+              aws.ec2.ami.search:
+                - resource_id: "string"
+                - executable_users
+                  - user1
+                - owners
+                  - amazon
+                - include_deprecated: True|False
+                - most_recent: True|False
+                - filters:
+                    - name: "string"
+                      values: "list"
+                    - name: "string"
+                      values: "list"
+
+    Examples:
+        .. code-block:: sls
 
             my-unmanaged-ami:
               aws.ec2.ami.search:
                 - resource_id: value
                 - most_recent: False
                 - filters:
-                    - name: 'name'
+                    - name: "name"
                       values: ["amzn-ami-minimal-hvm-2018.03.0.20181129-x86_64-ebs_2"]
     """
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
