@@ -43,7 +43,7 @@ async def present(
                 - policy_arn: arn:aws:iam::aws:policy/ReadOnlyAccess
 
     """
-    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
+    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     try:
         before_ret = (
             await hub.exec.aws.iam.role_policy_attachment.is_role_policy_attached(
@@ -51,32 +51,44 @@ async def present(
             )
         )
     except hub.tool.boto3.exception.ClientError as e:
-        result["comment"] = f"{e.__class__.__name__}: {e}"
+        result["comment"] = (f"{e.__class__.__name__}: {e}",)
         result["result"] = False
         return result
-    # If before_ret has False result and has an error message in comment, then immediately return with the error.
-    if before_ret["result"] is False and before_ret["comment"]:
-        result["result"] = False
-        result["comment"] = before_ret["comment"]
-        return result
 
-    if ctx.get("test", False):
-        if before_ret["result"]:
-            result["comment"] = f"aws.iam.role_policy_attachment {name} already exists"
-        else:
-            result["comment"] = f"Would attach aws.iam.role_policy_attachment {name}"
+    # If before_ret has False result and has an error message in comment, then immediately return with the error,
+    # unless this is a 'test' context - and if role does not exists - NoSuchEntityException is expected
+    if (
+        before_ret["result"] is False
+        and before_ret["comment"]
+        and not ctx.get("test", False)
+    ):
+        result["result"] = False
+        result["comment"] = before_ret["comment"]
         return result
 
     if before_ret["result"]:
         result[
             "old_state"
-        ] = hub.tool.aws.iam.utils.convert_raw_role_policy_attachment_to_present(
+        ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_policy_attachment_to_present(
             role_name, policy_arn
         )
         result["new_state"] = copy.deepcopy(result["old_state"])
-        result["comment"] = f"'{name}' already exists"
+        result["comment"] = (f"aws.iam.role_policy_attachment '{name}' already exists",)
     else:
         try:
+            if ctx.get("test", False):
+                result["new_state"] = hub.tool.aws.test_state_utils.generate_test_state(
+                    enforced_state={},
+                    desired_state={
+                        "role_name": role_name,
+                        "policy_arn": policy_arn,
+                    },
+                )
+                result["comment"] = (
+                    f"Would create aws.iam.role_policy_attachment. Role {role_name}, policy {policy_arn}",
+                )
+                return result
+
             ret = await hub.exec.boto3.client.iam.attach_role_policy(
                 ctx,
                 RoleName=role_name,
@@ -86,14 +98,14 @@ async def present(
             if not result["result"]:
                 result["comment"] = ret["comment"]
                 return result
-            result["comment"] = f"Attached '{name}'"
+            result["comment"] = (f"Created aws.iam.role_policy_attachment '{name}'",)
             result[
                 "new_state"
-            ] = hub.tool.aws.iam.utils.convert_raw_role_policy_attachment_to_present(
+            ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_policy_attachment_to_present(
                 role_name, policy_arn
             )
         except hub.tool.boto3.exception.ClientError as e:
-            result["comment"] = f"{e.__class__.__name__}: {e}"
+            result["comment"] = (f"{e.__class__.__name__}: {e}",)
             result["result"] = False
     return result
 
@@ -125,7 +137,7 @@ async def absent(
                 - role_name: value
                 - policy_arn: value
     """
-    result = dict(comment="", old_state=None, new_state=None, name=name, result=True)
+    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     try:
         before_ret = (
             await hub.exec.aws.iam.role_policy_attachment.is_role_policy_attached(
@@ -133,30 +145,35 @@ async def absent(
             )
         )
     except hub.tool.boto3.exception.ClientError as e:
-        result["comment"] = f"{e.__class__.__name__}: {e}"
+        result["comment"] = (f"{e.__class__.__name__}: {e}",)
         result["result"] = False
         return result
 
     if before_ret["result"] is False and before_ret["comment"]:
         if "NoSuchEntityException" in before_ret["comment"][0]:
-            result["comment"] = f"'{name}' already absent"
+            result["comment"] = (
+                f"aws.iam.role_policy_attachment '{name}' already absent",
+            )
         else:
             result["result"] = False
             result["comment"] = before_ret["comment"]
         return result
 
     if not before_ret["result"]:
-        result["comment"] = f"'{name}' already absent"
-    elif ctx.get("test", False):
-        result["comment"] = f"Would detach aws.iam.role_policy_attachment {name}"
-        return result
+        result["comment"] = (f"aws.iam.role_policy_attachment '{name}' already absent",)
     else:
         try:
             result[
                 "old_state"
-            ] = hub.tool.aws.iam.utils.convert_raw_role_policy_attachment_to_present(
+            ] = hub.tool.aws.iam.conversion_utils.convert_raw_role_policy_attachment_to_present(
                 role_name, policy_arn
             )
+            if ctx.get("test", False):
+                result["comment"] = (
+                    f"Would detach aws.iam.role_policy_attachment. Role '{role_name}', policy '{policy_arn}'",
+                )
+                return result
+
             ret = await hub.exec.boto3.client.iam.detach_role_policy(
                 ctx, RoleName=role_name, PolicyArn=policy_arn
             )
@@ -165,9 +182,11 @@ async def absent(
                 result["comment"] = ret["comment"]
                 result["result"] = False
                 return result
-            result["comment"] = f"Detached '{name}'"
+            result["comment"] = (
+                f"Detached aws.iam.role_policy_attachment. Role '{role_name}', policy '{policy_arn}'",
+            )
         except hub.tool.boto3.exception.ClientError as e:
-            result["comment"] = f"{e.__class__.__name__}: {e}"
+            result["comment"] = (f"{e.__class__.__name__}: {e}",)
     return result
 
 
@@ -214,7 +233,7 @@ async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
         else:
             resources = ret_attached_policies["ret"].get("AttachedPolicies")
             for resource in resources:
-                translated_resource = hub.tool.aws.iam.utils.convert_raw_role_policy_attachment_to_present(
+                translated_resource = hub.tool.aws.iam.conversion_utils.convert_raw_role_policy_attachment_to_present(
                     role_name, resource.get("PolicyArn")
                 )
                 resource_id = f"{role_name}-{translated_resource['policy_arn']}"
