@@ -1,4 +1,6 @@
 import copy
+from dataclasses import field
+from dataclasses import make_dataclass
 from typing import Any
 from typing import Dict
 from typing import List
@@ -13,10 +15,27 @@ async def present(
     hosted_zone_name: str,
     caller_reference: str,
     resource_id: str = None,
-    config: Dict = None,
-    vpcs: Dict = None,
+    vpcs: List[
+        make_dataclass(
+            "VPC",
+            [
+                ("VPCRegion", str, field(default=None)),
+                ("VPCId", str, field(default=None)),
+            ],
+        )
+    ] = None,
+    config: make_dataclass(
+        "HostedZoneConfig",
+        [
+            ("Comment", str, field(default=None)),
+            ("PrivateZone", bool, field(default=None)),
+        ],
+    ) = None,
     delegation_set_id: str = None,
-    tags: List = None,
+    tags: Dict[str, Any]
+    or List[
+        make_dataclass("Tag", [("Key", str), ("Value", str, field(default=None))])
+    ] = None,
 ) -> Dict[str, Any]:
     r"""
         Creates a new public or private hosted zone. You create records in a public hosted zone to define how you want
@@ -46,15 +65,21 @@ async def present(
                                       You must use a unique CallerReference string every time you submit a CreateHostedZone request.
                                       CallerReference can be any unique string, for example, a date/time stamp.,
             resource_id(Text, optional): AWS route53 hosted zone ID.
-            config(dictionary, optional):A complex type that contains the following optional values: Defaults to None
-                * Comment(Text) -- Any comments that you want to include about the hosted zone.
-                * PrivateZone(boolean) -- A value that indicates whether this is a private hosted zone.
-            vpcs(dictionary, optional): A complex type that contains information about the Amazon VPC that you're associating with
-                                 this hosted zone. Defaults to None
-                * VPCId(Text) -- (Private hosted zones only) The region that an Amazon VPC was created in.
-                * VPCRegion(Text) -- (Private hosted zones only) The ID of an Amazon VPC.
+            config(Dict[str, Any], optional): (Optional) A complex type that contains the following optional values:   For public and private
+                hosted zones, an optional comment   For private hosted zones, an optional PrivateZone element
+                If you don't specify a comment or the PrivateZone element, omit HostedZoneConfig and the other
+                elements. Defaults to None.
+                * Comment (str, optional): Any comments that you want to include about the hosted zone.
+                * PrivateZone (bool, optional): A value that indicates whether this is a private hosted zone.
+            vpcs(List, optional): (Private hosted zones only) A list that contains information about the Amazon VPC's that
+                you're associating with this hosted zone. If you are associating VPC's with a hosted zone with this request, the
+                paramaters VPCId and VPCRegion are also required. To associate additional Amazon VPCs with the
+                hosted zone, use AssociateVPCWithHostedZone after you create a hosted zone. Defaults to None.
+                * VPCRegion (str, optional): (Private hosted zones only) The region that an Amazon VPC was created in.
+                * VPCId (str, optional): (Private hosted zones only) The ID of an Amazon VPC.
             delegation_set_id(string, optional): Id of a reusable delegation set,
-            tags(List, optional): The tags to assign to the VPC. Defaults to None.
+            tags(List or Dict, optional): list of tags in the format of [{"Key": tag-key, "Value": tag-value}] or dict in the format of
+                {tag-key: tag-value} The tags to assign to the hosted zone. Defaults to None.
                 * Key (Text) -- The key of the tag. Tag keys are case-sensitive and accept a maximum of 127 Unicode characters. May not begin with aws: .
                 * Value (Text) -- The value of the tag. Tag values are case-sensitive and accept a maximum of 255 Unicode characters.
 
@@ -70,7 +95,7 @@ async def present(
                   PrivateZone: 'string'
                   Comment: 'string'
               - vpcs:
-                  VPCId: 'string'
+                - VPCId: 'string'
                   VPCRegion: 'string'
               - delegation_set_id: 'string'
     -         - tags:
@@ -94,8 +119,8 @@ async def present(
                         PrivateZone: True
                         Comment: 'description of hosted zone config'
                     - vpcs:
-                        VPCId: 'vpc id'
-                        VPCRegion: 'us-east1'
+                        - VPCId: 'vpc id'
+                          VPCRegion: 'us-east1'
                     - delegation_set_id: 'string'
     -               - tags:
                         - Key: 'tag key'
@@ -106,30 +131,16 @@ async def present(
     resource_updated = False
 
     if resource_id:
-        before = await hub.exec.boto3.client.route53.get_hosted_zone(
-            ctx, Id=resource_id
+        before = await hub.exec.aws.route53.hosted_zone.get(
+            ctx, name=name, resource_id=resource_id
         )
-
-    if before:
-        result["comment"] = (f"aws.route53.hostedzone '{name}' already exists.",)
-        old_tags = None
-        tags_ret = await hub.exec.boto3.client.route53.list_tags_for_resource(
-            ctx, ResourceType="hostedzone", ResourceId=resource_id
-        )
-        if tags_ret["result"]:
-            old_tags = tags_ret["ret"].get("ResourceTagSet").get("Tags", [])
-        else:
+        if not before["result"] or not before["ret"]:
             result["result"] = False
-            result["comment"] = tags_ret["comment"]
+            result["comment"] = before["comment"]
             return result
 
-        result[
-            "old_state"
-        ] = hub.tool.aws.route53.conversion_utils.convert_raw_hosted_zone_to_present(
-            raw_resource=before,
-            idem_resource_name=name,
-            tags=old_tags,
-        )
+        result["old_state"] = copy.deepcopy(before["ret"])
+        result["comment"] = (f"aws.route53.hosted_zone '{name}' already exists.",)
         plan_state = copy.deepcopy(result["old_state"])
         # Check if comments needs to be updated.
         if (
@@ -139,7 +150,7 @@ async def present(
         ):
             if ctx.get("test", False):
                 result["comment"] = result["comment"] + (
-                    f"Would update aws.route53.hostedzone '{name}'",
+                    f"Would update aws.route53.hosted_zone '{name}'",
                 )
                 resource_updated = True
                 plan_state["config"] = config
@@ -160,6 +171,7 @@ async def present(
                 else:
                     result["comment"] = update_ret_comment["comment"]
                     result["result"] = False
+                    return result
 
         # Check if tags needs to be updated
         if tags is not None:
@@ -167,7 +179,7 @@ async def present(
                 ctx,
                 resource_type="hostedzone",
                 resource_id=resource_id,
-                old_tags=old_tags,
+                old_tags=result["old_state"]["tags"],
                 new_tags=tags,
             )
             result["comment"] = result["comment"] + update_ret_tags["comment"]
@@ -176,8 +188,14 @@ async def present(
                 return result
             resource_updated = resource_updated or bool(update_ret_tags["ret"])
             if ctx.get("test", False) and update_ret_tags["ret"]:
-                plan_state["tags"] = update_ret_tags["ret"].get("tags")
+                plan_state["tags"] = update_ret_tags["ret"]
     else:
+        tags_dict = None
+        if tags is not None:
+            if isinstance(tags, List):
+                tags_dict = hub.tool.aws.tag_utils.convert_tag_list_to_dict(tags)
+            else:
+                tags_dict = tags
         if ctx.get("test", False):
             result["new_state"] = hub.tool.aws.test_state_utils.generate_test_state(
                 enforced_state={},
@@ -188,7 +206,7 @@ async def present(
                     "config": config,
                     "vpcs": vpcs,
                     "delegation_set_id": delegation_set_id,
-                    "tags": tags,
+                    "tags": tags_dict,
                 },
             )
             result["comment"] = hub.tool.aws.comment_utils.would_create_comment(
@@ -196,16 +214,23 @@ async def present(
             )
             return result
 
+        # Private Route53 Hosted Zones can only be created with their first VPC association,
+        # however we need to associate the remaining after creation.
+
+        vpc = None
+        if vpcs is not None and len(vpcs) > 0:
+            vpc = vpcs[0]
+
         ret = await hub.exec.boto3.client.route53.create_hosted_zone(
             ctx,
             Name=hosted_zone_name,
-            VPC=vpcs,
+            VPC=vpc,
             CallerReference=caller_reference,
             HostedZoneConfig=config,
             DelegationSetId=delegation_set_id,
         )
-        result["result"] = ret["result"]
-        if not result["result"]:
+        if not ret["result"]:
+            result["result"] = False
             result["comment"] = ret["comment"]
             return result
 
@@ -213,40 +238,48 @@ async def present(
             resource_type="aws.route53.hosted_zone", name=name
         )
         resource_id = ret["ret"]["HostedZone"]["Id"].split("/")[-1]
+        if vpcs is not None and len(vpcs) > 1:
+            for i in range(1, len(vpcs)):
+                vpc_ret = (
+                    await hub.exec.boto3.client.route53.associate_vpc_with_hosted_zone(
+                        ctx, HostedZoneId=resource_id, VPC=vpcs[i]
+                    )
+                )
+                if not vpc_ret["result"]:
+                    result["result"] = False
+                    result["comment"] = vpc_ret["comment"]
+                    hub.log.debug(
+                        f"Could not associate vpc with hosted zone {vpc_ret['comment']}"
+                    )
+                    return result
         if tags is not None:
-            # add tag
             add_tags = await hub.exec.aws.route53.tag.update_tags(
                 ctx,
                 resource_type="hostedzone",
                 resource_id=resource_id,
                 old_tags=None,
-                new_tags=tags,
+                new_tags=tags_dict,
             )
             if not add_tags["result"]:
                 error = add_tags["comment"]
                 result["result"] = False
-                hub.log.debug(f"Tag addition failed for {name} with error {error}")
-                result["comment"] = result["comment"] + add_tags["comment"]
+                hub.log.debug(
+                    f"Tag addition failed for hosted_zone {name} with error {error}"
+                )
+                result["comment"] = result["comment"] + error
+                return result
 
     if ctx.get("test", False):
         result["new_state"] = plan_state
-    elif (not before) or resource_updated:
-        after = await hub.exec.boto3.client.route53.get_hosted_zone(ctx, Id=resource_id)
-        if after["result"]:
-            tags_ret = await hub.exec.boto3.client.route53.list_tags_for_resource(
-                ctx,
-                ResourceType="hostedzone",
-                ResourceId=resource_id,
-            )
-            if tags_ret["result"]:
-                tags = tags_ret["ret"]["ResourceTagSet"].get("Tags")
-            result[
-                "new_state"
-            ] = hub.tool.aws.route53.conversion_utils.convert_raw_hosted_zone_to_present(
-                raw_resource=after,
-                idem_resource_name=name,
-                tags=tags,
-            )
+    elif result["old_state"] is None or resource_updated:
+        after = await hub.exec.aws.route53.hosted_zone.get(
+            ctx=ctx, name=name, resource_id=resource_id
+        )
+        if not after["result"]:
+            result["result"] = False
+            result["comment"] = after["comment"]
+            return result
+        result["new_state"] = copy.deepcopy(after["ret"])
     else:
         result["new_state"] = copy.deepcopy(result["old_state"])
     return result
@@ -309,37 +342,32 @@ async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]
             resource_type="aws.route53.hosted_zone", name=name
         )
         return result
-    before = await hub.exec.boto3.client.route53.get_hosted_zone(ctx, Id=resource_id)
-
+    before = await hub.exec.aws.route53.hosted_zone.get(
+        ctx, name=name, resource_id=resource_id
+    )
     if not before["result"]:
+        result["comment"] = before["comment"]
+        result["result"] = False
+        return result
+    if not before["ret"]:
         result["comment"] = hub.tool.aws.comment_utils.already_absent_comment(
             resource_type="aws.route53.hosted_zone", name=name
         )
+    elif ctx.get("test", False):
+        result["old_state"] = before["ret"]
+        result["comment"] = hub.tool.aws.comment_utils.would_delete_comment(
+            resource_type="aws.route53.hosted_zone", name=name
+        )
+        return result
     else:
-        tags = None
-        tags_ret = await hub.exec.boto3.client.route53.list_tags_for_resource(
-            ctx, ResourceType="hostedzone", ResourceId=resource_id
-        )
-        if tags_ret["result"]:
-            tags = tags_ret["ret"]["ResourceTagSet"].get("Tags")
-        result[
-            "old_state"
-        ] = hub.tool.aws.route53.conversion_utils.convert_raw_hosted_zone_to_present(
-            raw_resource=before,
-            idem_resource_name=name,
-            tags=tags,
-        )
-        if ctx.get("test", False):
-            result["comment"] = hub.tool.aws.comment_utils.would_delete_comment(
-                resource_type="aws.route53.hosted_zone", name=name
-            )
-            return result
+        result["old_state"] = before["ret"]
         ret = await hub.exec.boto3.client.route53.delete_hosted_zone(
             ctx, Id=resource_id
         )
         result["result"] = ret["result"]
         if not result["result"]:
             result["comment"] = ret["comment"]
+            result["result"] = False
             return result
         result["comment"] = hub.tool.aws.comment_utils.delete_comment(
             resource_type="aws.route53.hosted_zone", name=name
@@ -377,30 +405,23 @@ async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
         return result
 
     for hosted_zone in ret["ret"]["HostedZones"]:
-        tags = None
         resource_id = hosted_zone.get("Id")
+        name = hosted_zone.get("Name")
         resource_id = resource_id.split("/")[-1]
-        individual_hosted_zone = await hub.exec.boto3.client.route53.get_hosted_zone(
-            ctx, Id=resource_id
+        resource = await hub.exec.aws.route53.hosted_zone.get(
+            ctx, name=name, resource_id=resource_id
         )
 
-        tags_ret = await hub.exec.boto3.client.route53.list_tags_for_resource(
-            ctx, ResourceType="hostedzone", ResourceId=resource_id
-        )
-        if tags_ret["result"]:
-            tags = tags_ret["ret"]["ResourceTagSet"].get("Tags")
-
-        resource_translated = (
-            hub.tool.aws.route53.conversion_utils.convert_raw_hosted_zone_to_present(
-                raw_resource=individual_hosted_zone,
-                idem_resource_name=resource_id,
-                tags=tags,
+        if not resource["result"]:
+            hub.log.info(
+                f"Failed to retrieve aws.route53.hosted_zone {name}: {resource['comment']}"
             )
-        )
+            continue
+
         result[resource_id] = {
             "aws.route53.hosted_zone.present": [
                 {parameter_key: parameter_value}
-                for parameter_key, parameter_value in resource_translated.items()
+                for parameter_key, parameter_value in resource["ret"].items()
             ]
         }
 
@@ -455,9 +476,16 @@ async def search(
                     Value: idem-test-hosted_zone
 
     """
+
+    hub.log.warning(
+        f"aws.route53.hosted_zone.search '{name}' state has been deprecated. Please use exec.run with aws.route53.hosted_zone.get instead."
+    )
     result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)
     if resource_id:
-        ret = await hub.exec.boto3.client.route53.get_hosted_zone(ctx, Id=resource_id)
+        # If resource_id is mentioned directly returning the hosted_zone
+        ret = await hub.exec.aws.route53.hosted_zone.get(
+            ctx, name=name, resource_id=resource_id
+        )
         if not ret["result"]:
             result["result"] = ret["result"]
             result["comment"] = (
@@ -477,29 +505,32 @@ async def search(
             return result
         tags = tags_ret["ret"]["ResourceTagSet"].get("Tags")
     else:
-        describe_ret = await hub.states.aws.route53.hosted_zone.describe(ctx)
-        if not describe_ret:
+        # Get all the hosted_zones to apply the filters
+        describe_ret = (
+            await hub.tool.aws.route53.hosted_zone_utils.get_all_hosted_zones(ctx)
+        )
+        if not describe_ret["result"]:
             result["result"] = False
             result["comment"] = (
-                "Could not describe all the aws.route53.hosted_zone resources to apply the filters",
+                f"Could not describe all the aws.route53.hosted_zone resources to apply the filters, {describe_ret['comment']}",
             )
             return result
 
-        ret = hub.tool.aws.route53.route53_utils.get_hosted_zone_with_filters(
-            ctx,
-            raw_hosted_zones=describe_ret,
+        ret = hub.tool.aws.route53.hosted_zone_utils.get_hosted_zones_with_filters(
+            raw_hosted_zones=describe_ret["ret"],
             hosted_zone_name=hosted_zone_name,
             private_zone=private_zone,
             vpc_id=vpc_id,
             tags=tags,
         )
-        result["comment"] = ret["comment"]
-        if ret["ret"] is not None:
-            result["old_state"] = ret["ret"]
-            result["old_state"]["name"] = name
-            result["new_state"] = copy.deepcopy(result["old_state"])
+        result["comment"] = result["comment"] + ret["comment"]
+        if not ret["ret"]:
+            result["result"] = ret["result"]
+            return result
 
-        return result
+        # For backward compatibility since search returns a single object
+        hosted_zone = ret["ret"][0]
+        tags = hosted_zone.get("tags")
 
     result[
         "old_state"
