@@ -14,13 +14,16 @@ async def convert_raw_vpc_to_present(
     resource_parameters = OrderedDict(
         {
             "InstanceTenancy": "instance_tenancy",
-            "Tags": "tags",
         }
     )
     resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
     for parameter_raw, parameter_present in resource_parameters.items():
         if parameter_raw in raw_resource:
             resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     # The following code block is to make sure to only output the associated/associating cidr blocks and
     # dis-regard the disassociated cidr blocks.
     if raw_resource.get("CidrBlockAssociationSet"):
@@ -98,13 +101,16 @@ def convert_raw_subnet_to_present(
             "EnableDns64": "enable_dns_64",
             "PrivateDnsNameOptionsOnLaunch": "private_dns_name_options_on_launch",
             "EnableLniAtDeviceIndex": "enable_lni_at_device_index",
-            "Tags": "tags",
         }
     )
     resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
     for parameter_raw, parameter_present in resource_parameters.items():
         if parameter_raw in raw_resource:
             resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     if (not raw_resource.get("AvailabilityZone")) and raw_resource.get(
         "AvailabilityZoneId"
     ):
@@ -140,7 +146,6 @@ def convert_raw_sg_rule_to_present(
             "ToPort": "to_port",
             "CidrIpv4": "cidr_ipv4",
             "CidrIpv6": "cidr_ipv6",
-            "Tags": "tags",
             "PrefixListId": "prefix_list_id",
             "Description": "description",
             "ReferencedGroupInfo": "referenced_group_info",
@@ -150,6 +155,10 @@ def convert_raw_sg_rule_to_present(
     for parameter_raw, parameter_present in resource_parameters.items():
         if parameter_raw in raw_resource:
             resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     return resource_translated
 
 
@@ -162,14 +171,84 @@ def convert_raw_transit_gateway_to_present(
             "State": "state",
             "Description": "description",
             "Options": "options",
-            "Tags": "tags",
         }
     )
     resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
     for parameter_raw, parameter_present in resource_parameters.items():
         if parameter_raw in raw_resource:
             resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+    return resource_translated
 
+
+def convert_raw_key_pair_to_present(
+    hub, raw_resource: Dict[str, Any]
+) -> Dict[str, Any]:
+    """
+    Convert the aws key pair response to a common format
+
+    @param raw_resource: describe key pair response
+    @param hub: required for functions in hub
+
+    @return:
+        A dictionary key pair's properties
+    """
+    if not raw_resource:
+        return {}
+
+    resource_parameters = OrderedDict(
+        {
+            "KeyPairId": "resource_id",
+            "PublicKey": "public_key",
+        }
+    )
+    resource_translated = {"name": raw_resource.get("KeyName")}
+    for raw_parameter, present_parameter in resource_parameters.items():
+        if raw_resource.get(raw_parameter):
+            resource_translated[present_parameter] = raw_resource.get(raw_parameter)
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+    return resource_translated
+
+
+def convert_raw_placement_group_to_present(
+    hub, raw_resource: Dict[str, Any]
+) -> Dict[str, Any]:
+    """
+    Convert the aws placement_group response to a common format
+
+    Args:
+        hub: required for functions in hub
+        raw_resource: The aws response to convert
+
+    Returns: Valid idem state for placement_group of type Dict['string', Any]
+    """
+    if not raw_resource:
+        return {}
+
+    resource_parameters = OrderedDict(
+        {
+            "State": "state",
+            "Strategy": "strategy",
+            "PartitionCount": "partition_count",
+            "GroupId": "group_id",
+            "GroupName": "group_name",
+            "SpreadLevel": "spread_level",
+        }
+    )
+    resource_translated = {"resource_id": raw_resource.get("GroupName", None)}
+    for raw_parameter, present_parameter in resource_parameters.items():
+        if raw_resource.get(raw_parameter):
+            resource_translated[present_parameter] = raw_resource.get(raw_parameter)
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     return resource_translated
 
 
@@ -193,7 +272,6 @@ def convert_raw_flow_log_to_present(hub, flow_log: Dict):
             "LogDestination": "log_destination",
             "LogFormat": "log_format",
             "MaxAggregationInterval": "max_aggregation_interval",
-            "Tags": "tags",
         }
     )
     translated_flow_log = {}
@@ -214,45 +292,56 @@ def convert_raw_flow_log_to_present(hub, flow_log: Dict):
     for parameter_old_key, parameter_new_key in describe_parameters.items():
         if flow_log.get(parameter_old_key) is not None:
             translated_flow_log[parameter_new_key] = flow_log.get(parameter_old_key)
-
+    if flow_log.get("Tags") is not None:
+        translated_flow_log["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            flow_log.get("Tags")
+        )
     return translated_flow_log
 
 
 def convert_raw_route_table_to_present(
-    hub, resource: Dict, idem_resource_name: str = None
+    hub, raw_resource: Dict, idem_resource_name: str = None
 ) -> Dict[str, Any]:
     describe_parameters = OrderedDict(
-        {
-            "VpcId": "vpc_id",
-            "PropagatingVgws": "propagating_vgws",
-            "Tags": "tags",
-        }
+        {"VpcId": "vpc_id", "PropagatingVgws": "propagating_vgws", "Tags": "tags"}
     )
-    if not resource:
+    if not raw_resource:
         return {}
-    resource_id = resource.get("RouteTableId")
+    resource_id = raw_resource.get("RouteTableId")
     resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
     for parameter_old_key, parameter_new_key in describe_parameters.items():
-        if resource.get(parameter_old_key) is not None:
-            resource_translated[parameter_new_key] = resource.get(parameter_old_key)
-
-    # Add only required parameters in Describe
+        if raw_resource.get(parameter_old_key) is not None:
+            resource_translated[parameter_new_key] = raw_resource.get(parameter_old_key)
     routes_to_add = []
-    if resource.get("Routes"):
-        for route_to_add in resource.get("Routes"):
+    if raw_resource.get("Routes"):
+        for route_to_add in raw_resource.get("Routes"):
             if route_to_add.get("State") == "active":
                 routes_to_add.append(route_to_add)
         resource_translated["routes"] = routes_to_add
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+    return resource_translated
 
-    associations_to_add = []
-    if resource.get("Associations"):
-        for association_to_add in resource.get("Associations"):
-            if (
-                association_to_add.get("AssociationState").get("State") == "associated"
-                and not association_to_add["Main"]
-            ):
-                associations_to_add.append(association_to_add)
-        resource_translated["associations"] = associations_to_add
+
+def convert_raw_route_table_association_to_present(
+    hub, resource: Dict, idem_resource_name: str = None
+) -> Dict[str, Any]:
+    if not resource:
+        return {}
+    resource_id = resource.get("RouteTableAssociationId")
+    resource_parameters = OrderedDict(
+        {
+            "RouteTableId": "route_table_id",
+            "GatewayId": "gateway_id",
+            "SubnetId": "subnet_id",
+        }
+    )
+    resource_translated = {"name": idem_resource_name, "resource_id": resource_id}
+    for parameter_raw, parameter_present in resource_parameters.items():
+        if parameter_raw in resource:
+            resource_translated[parameter_present] = resource.get(parameter_raw)
     return resource_translated
 
 
@@ -268,7 +357,6 @@ def convert_raw_tg_vpc_attachment(
             "VpcId": "vpc_id",
             "State": "state",
             "SubnetIds": "subnet_ids",
-            "Tags": "tags",
             "Options": "options",
         }
     )
@@ -276,6 +364,10 @@ def convert_raw_tg_vpc_attachment(
     for camel_case_key, snake_case_key in resource_parameters_transit_gateway.items():
         if raw_resource.get(camel_case_key):
             translated_resource[snake_case_key] = raw_resource.get(camel_case_key)
+    if raw_resource.get("Tags") is not None:
+        translated_resource["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     return translated_resource
 
 
@@ -314,10 +406,39 @@ async def convert_raw_dhcp_to_present(
     if vpc_list:
         resource_translated["vpc_id"] = vpc_list
 
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     result["ret"] = resource_translated
     return result
 
 
+def convert_raw_dhcp_association_to_present(
+    hub, dhcp_id: str, idem_resource_name: str = None, vpc_id: str = None
+) -> Dict[str, Any]:
+    """
+    Convert the AWS EC2 DHCP Options Association response to a common format
+
+    Args:
+        hub: required for functions in hub
+        dhcp_id: ID of DHCP Options.
+        idem_resource_name: name of the resource
+        vpc_id: ID of VPC to which DHCP options need to be attached.
+
+    Returns:
+        A dictionary of EC2 DHCP options.
+    """
+    resource_translated = {
+        "name": idem_resource_name,
+        "dhcp_id": dhcp_id,
+        "resource_id": f"{dhcp_id}-{vpc_id}",
+    }
+    if vpc_id:
+        resource_translated["vpc_id"] = vpc_id
+    return resource_translated
+
+
 def convert_raw_sg_to_present(hub, security_group: Dict):
     """
     Convert the aws security group response to a common format
@@ -335,7 +456,6 @@ def convert_raw_sg_to_present(hub, security_group: Dict):
             "GroupId": "resource_id",
             "GroupName": "name",
             "VpcId": "vpc_id",
-            "Tags": "tags",
             "Description": "description",
         }
     )
@@ -344,6 +464,10 @@ def convert_raw_sg_to_present(hub, security_group: Dict):
             translated_security_group[parameter_new_key] = security_group.get(
                 parameter_old_key
             )
+    if "Tags" in security_group:
+        translated_security_group[
+            "tags"
+        ] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(security_group.get("Tags"))
 
     return translated_security_group
 
@@ -360,6 +484,10 @@ def convert_raw_internet_gateway_to_present(
     if resource.get("Attachments"):
         resource_translated["vpc_id"] = [resource.get("Attachments")[0].get("VpcId")]
         resource_translated["attachments"] = resource.get("Attachments")
+    if resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            resource.get("Tags")
+        )
     return resource_translated
 
 
@@ -371,7 +499,6 @@ def convert_raw_nat_gateway_to_present(
         {
             "SubnetId": "subnet_id",
             "ConnectivityType": "connectivity_type",
-            "Tags": "tags",
             "State": "state",
         }
     )
@@ -386,6 +513,10 @@ def convert_raw_nat_gateway_to_present(
                     "AllocationId"
                 )
                 break
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     return resource_translated
 
 
@@ -398,7 +529,7 @@ def convert_raw_elastic_ip_to_present(
     Args:
         hub: required for functions in hub
         raw_resource (Dict): The dictionary object from where the raw state of resource needs to be translated.
-        idem_resource_name (Text): The idem of the idem resource
+        idem_resource_name (Text): The name of the idem resource
 
     Returns: Dict[str, Any]
     """
@@ -408,7 +539,9 @@ def convert_raw_elastic_ip_to_present(
     describe_parameters = OrderedDict(
         {
             "AllocationId": "allocation_id",
+            "AssociationId": "association_id",
             "Domain": "domain",
+            "InstanceId": "instance_id",
             "NetworkBorderGroup": "network_border_group",
             "PublicIpv4Pool": "public_ipv4_pool",
             "Tags": "tags",
@@ -419,6 +552,10 @@ def convert_raw_elastic_ip_to_present(
     for parameter_raw, parameter_present in describe_parameters.items():
         if elastic_ip.get(parameter_raw) is not None:
             resource_translated[parameter_present] = elastic_ip.get(parameter_raw)
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     return resource_translated
 
 
@@ -447,7 +584,7 @@ def convert_raw_vpc_endpoint_to_present(
             "SubnetIds": "subnet_ids",
             "SecurityGroupIds": "security_group_ids",
             "PrivateDnsEnabled": "private_dns_enabled",
-            "Tags": "tags",
+            "State": "state",
         }
     )
     resource_id = raw_resource.get("VpcEndpointId")
@@ -455,6 +592,10 @@ def convert_raw_vpc_endpoint_to_present(
     for parameter_raw, parameter_present in describe_parameters.items():
         if raw_resource.get(parameter_raw) is not None:
             resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+    if raw_resource.get("Tags") is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
 
     return resource_translated
 
@@ -479,7 +620,6 @@ def convert_raw_ami_to_present(hub, raw_resource: Dict[str, Any]) -> Dict[str, A
             "SriovNetSupport": "sriov_net_support",
             "VirtualizationType": "virtualization_type",
             "BootMode": "boot_mode",
-            "Tags": "tags",
         }
     )
     resource_translated = {"resource_id": resource_id}
@@ -487,4 +627,137 @@ def convert_raw_ami_to_present(hub, raw_resource: Dict[str, Any]) -> Dict[str, A
         if raw_resource.get(parameter_raw):
             resource_translated[parameter_present] = raw_resource.get(parameter_raw)
 
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+    return resource_translated
+
+
+def convert_raw_vpc_peering_connection_to_present(
+    hub, raw_resource: Dict[str, Any], idem_resource_name: str
+) -> Dict[str, Any]:
+    if not raw_resource:
+        return {}
+
+    resource_translated = {
+        "resource_id": raw_resource["VpcPeeringConnectionId"],
+        "name": idem_resource_name,
+        "peer_owner_id": raw_resource["AccepterVpcInfo"]["OwnerId"],
+        "peer_vpc_id": raw_resource["AccepterVpcInfo"]["VpcId"],
+        "vpc_id": raw_resource["RequesterVpcInfo"]["VpcId"],
+        "peer_region": raw_resource["AccepterVpcInfo"]["Region"],
+        "tags": hub.tool.aws.tag_utils.convert_tag_list_to_dict(raw_resource["Tags"]),
+        "status": raw_resource["Status"]["Code"],
+    }
+
+    return resource_translated
+
+
+def convert_raw_availability_zone_to_snake_case(
+    hub, ctx, raw_resource: Dict[str, Any]
+) -> Dict[str, Any]:
+    """
+    Util function to convert raw resource state from AWS Availability Zone to a snake case format.
+    """
+    if not raw_resource:
+        return {}
+
+    resource_parameters = OrderedDict(
+        {
+            "GroupName": "group_name",
+            "NetworkBorderGroup": "network_border_group",
+            "OptInStatus": "opt_in_status",
+            "ParentZoneId": "parent_zone_id",
+            "ParentZoneName": "parent_zone_name",
+            "RegionName": "region_name",
+            "State": "state",
+            "ZoneName": "zone_name",
+            "ZoneType": "zone_type",
+        }
+    )
+    resource_translated = {"resource_id": raw_resource["ZoneId"]}
+    for parameter_raw, parameter_present in resource_parameters.items():
+        if raw_resource.get(parameter_raw):
+            resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+
+    return resource_translated
+
+
+async def convert_raw_snapshot_to_present(
+    hub, ctx, raw_resource: Dict[str, Any], idem_resource_name: str
+) -> Dict[str, Any]:
+    """
+    Util function to convert raw resource state from AWS snapshot to present input format
+    """
+    resource_id = raw_resource.get("SnapshotId")
+    resource_parameters = OrderedDict(
+        {
+            "Description": "description",
+            "OutpostArn": "outpost_arn",
+            "Tags": "tags",
+            "VolumeId": "volume_id",
+        }
+    )
+    resource_translated = {"resource_id": resource_id}
+    resource_translated["name"] = idem_resource_name
+    for parameter_raw, parameter_present in resource_parameters.items():
+        if raw_resource.get(parameter_raw):
+            resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+
+    """
+        Expansion of the volume associated with the snapshot, if present. This is due to the one-way
+        relationship with volume once the snapshot is created, to allow the user access to up to date
+        values of the volume if it has not been removed. This also fills a feature gap with terraform
+        which provides some of this expansion as part of its snapshot support.
+    """
+    if resource_translated["volume_id"] is not None:
+        volume = await hub.exec.aws.ec2.volume.get(
+            ctx=ctx,
+            resource_id=resource_translated["volume_id"],
+            name=resource_translated["name"],
+        )
+
+    if volume["result"] and volume["ret"]:
+        resource_translated["associated_volume"] = volume["ret"]
+
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+
+    return resource_translated
+
+
+def convert_raw_volume_to_present(
+    hub, raw_resource: Dict[str, Any], idem_resource_name: str
+) -> Dict[str, Any]:
+    """
+    Util function to convert raw resource state from AWS Volume to present input format
+    """
+    resource_id = raw_resource.get("VolumeId")
+    resource_parameters = OrderedDict(
+        {
+            "AvailabilityZone": "availability_zone",
+            "Size": "size",
+            "SnapshotId": "snapshot_id",
+            "State": "state",
+            "Iops": "iops",
+            "Tags": "tags",
+            "VolumeType": "volume_type",
+            "MultiAttachEnabled": "multi_attach_enabled",
+            "Throughput": "throughput",
+            "Encrypted": "encrypted",
+        }
+    )
+    resource_translated = {"resource_id": resource_id}
+    resource_translated["name"] = idem_resource_name
+    for parameter_raw, parameter_present in resource_parameters.items():
+        if raw_resource.get(parameter_raw):
+            resource_translated[parameter_present] = raw_resource.get(parameter_raw)
+
+    if "Tags" in raw_resource:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
     return resource_translated
