@@ -12,7 +12,6 @@ def convert_raw_instance_profile_to_present(
             "InstanceProfileName": "name",
             "InstanceProfileId": "id",
             "Path": "path",
-            "Tags": "tags",
         }
     )
 
@@ -21,6 +20,10 @@ def convert_raw_instance_profile_to_present(
     for parameter_raw, parameter_present in resource_parameters.items():
         if parameter_raw in raw_resource:
             translated_resource[parameter_present] = raw_resource.get(parameter_raw)
+    if raw_resource.get("Tags") is not None:
+        translated_resource["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
 
     translated_resource["roles"] = []
     for role in raw_resource.get("Roles"):
@@ -83,6 +86,10 @@ async def convert_raw_policy_to_present(
         if parameter_raw in raw_resource:
             translated_resource[parameter_present] = raw_resource.get(parameter_raw)
 
+    if translated_resource.get("tags") is not None:
+        translated_resource["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            translated_resource.get("tags")
+        )
     return translated_resource
 
 
@@ -95,7 +102,6 @@ def convert_raw_role_to_present(hub, raw_resource: Dict[str, Any]) -> Dict[str,
             "Path": "path",
             "Description": "description",
             "MaxSessionDuration": "max_session_duration",
-            "Tags": "tags",
         }
     )
 
@@ -105,6 +111,11 @@ def convert_raw_role_to_present(hub, raw_resource: Dict[str, Any]) -> Dict[str,
         if parameter_raw in raw_resource:
             translated_resource[parameter_present] = raw_resource.get(parameter_raw)
 
+    if raw_resource.get("Tags") is not None:
+        translated_resource["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+
     if raw_resource.get("PermissionsBoundary"):
         translated_resource["permissions_boundary"] = raw_resource.get(
             "PermissionsBoundary"
@@ -190,7 +201,6 @@ def convert_raw_user_to_present(
             "Arn": "arn",
             "Path": "path",
             "PermissionsBoundary": "permissions_boundary",
-            "Tags": "tags",
             "UserName": "user_name",
         }
     )
@@ -202,6 +212,11 @@ def convert_raw_user_to_present(
         if parameter_raw in raw_resource:
             translated_resource[parameter_present] = raw_resource.get(parameter_raw)
 
+    if raw_resource.get("Tags") is not None:
+        translated_resource["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
+
     return translated_resource
 
 
@@ -216,13 +231,16 @@ def convert_raw_oidc_provider_to_present(
             "Url": "url",
             "ClientIDList": "client_id_list",
             "ThumbprintList": "thumbprint_list",
-            "Tags": "tags",
         }
     )
     translated_resource = {"name": idem_resource_name, "resource_id": resource_id}
     for parameter_raw, parameter_present in resource_parameters.items():
         if parameter_raw in raw_resource:
             translated_resource[parameter_present] = raw_resource.get(parameter_raw)
+    if raw_resource.get("Tags") is not None:
+        translated_resource["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            raw_resource.get("Tags")
+        )
 
     translated_resource["url"] = "https://" + raw_resource.get("Url")
     return translated_resource
@@ -317,3 +335,30 @@ def convert_access_key_to_present(hub, access_key, idem_resource_name: str):
             translated_resource[new_k] = access_key[api_k]
 
     return translated_resource
+
+
+def convert_raw_password_policy_to_present(
+    hub, ctx, raw_password_policy: Dict[str, Any]
+) -> Dict[str, Any]:
+    resource_parameters = OrderedDict(
+        {
+            "MinimumPasswordLength": "minimum_password_length",
+            "RequireSymbols": "require_symbols",
+            "RequireNumbers": "require_numbers",
+            "RequireUppercaseCharacters": "require_uppercase_characters",
+            "RequireLowercaseCharacters": "require_lowercase_characters",
+            "AllowUsersToChangePassword": "allow_users_to_change_password",
+            "ExpirePasswords": "expire_passwords",
+            "MaxPasswordAge": "max_password_age",
+            "PasswordReusePrevention": "password_reuse_prevention",
+            "HardExpiry": "hard_expiry",
+        }
+    )
+    resource_translated = {}
+    for parameter_raw, parameter_present in resource_parameters.items():
+        if parameter_raw in raw_password_policy:
+            resource_translated[parameter_present] = raw_password_policy.get(
+                parameter_raw
+            )
+
+    return resource_translated
