@@ -98,7 +98,10 @@ def convert_raw_hosted_zone_to_present(
         resource_translated["vpcs"] = vpcs
     if delegation_set is not None:
         resource_translated["delegation_set"] = delegation_set.copy()
-    resource_translated["tags"] = tags
+    if tags is not None:
+        resource_translated["tags"] = hub.tool.aws.tag_utils.convert_tag_list_to_dict(
+            tags
+        )
 
     return resource_translated
 
