@@ -1,5 +1,5 @@
+import asyncio
 import re
-import time
 from typing import Any
 from typing import Dict
 from typing import Generator
@@ -33,15 +33,7 @@ async def exec_(
     """
     session: boto3.session.Session = hub.tool.boto3.session.get()
     client = session.client(
-        service_name=service_name,
-        region_name=ctx.acct.get("region_name"),
-        api_version=ctx.acct.get("api_version"),
-        use_ssl=ctx.acct.get("use_ssl", True),
-        endpoint_url=ctx.acct.get("endpoint_url"),
-        aws_access_key_id=ctx.acct.get("aws_access_key_id"),
-        aws_secret_access_key=ctx.acct.get("aws_secret_access_key"),
-        aws_session_token=ctx.acct.get("aws_session_token"),
-        verify=ctx.acct.get("verify"),
+        service_name=service_name, **hub.tool.boto3.session.kwargs(ctx)
     )
 
     # Don't pass kwargs that have a "None" value to the function call
@@ -67,15 +59,7 @@ async def exec_(
 def get_client(hub, ctx, service_name: str):
     session: boto3.session.Session = hub.tool.boto3.session.get()
     client = session.client(
-        service_name=service_name,
-        region_name=ctx.acct.get("region_name"),
-        api_version=ctx.acct.get("api_version"),
-        use_ssl=ctx.acct.get("use_ssl", True),
-        endpoint_url=ctx.acct.get("endpoint_url"),
-        aws_access_key_id=ctx.acct.get("aws_access_key_id"),
-        aws_secret_access_key=ctx.acct.get("aws_secret_access_key"),
-        aws_session_token=ctx.acct.get("aws_session_token"),
-        verify=ctx.acct.get("verify"),
+        service_name=service_name, **hub.tool.boto3.session.kwargs(ctx)
     )
     return client
 
@@ -109,7 +93,7 @@ async def wait(
         if waiter is None:
             raise NameError(f"No custom waiter object defined for '{waiter_name}'. ")
     else:
-        client = get_client(hub, ctx, service_name)
+        client = hub.tool.boto3.client.get_client(ctx, service_name)
 
         if waiter_name not in client.waiter_names:
             raise NameError(
@@ -122,7 +106,7 @@ async def wait(
     if delay is not None:
         # There might be some delay for the resource to start updating/deleting after the api call
         hub.log.debug(f"Waiting '{delay}' seconds, before starting the waiter")
-        time.sleep(delay)
+        await asyncio.sleep(delay)
 
     await hub.pop.loop.wrap(waiter.wait, *wt_args, **wt_kwargs)
 
@@ -138,15 +122,7 @@ def search(hub, ctx, service_name: str, collection: str, jmes_path: str = "*[]")
     """
     session: boto3.session.Session = hub.tool.boto3.session.get()
     client = session.client(
-        service_name=service_name,
-        region_name=ctx.acct.get("region_name"),
-        api_version=ctx.acct.get("api_version"),
-        use_ssl=ctx.acct.get("use_ssl", True),
-        endpoint_url=ctx.acct.get("endpoint_url"),
-        aws_access_key_id=ctx.acct.get("aws_access_key_id"),
-        aws_secret_access_key=ctx.acct.get("aws_secret_access_key"),
-        aws_session_token=ctx.acct.get("aws_session_token"),
-        verify=ctx.acct.get("verify"),
+        service_name=service_name, **hub.tool.boto3.session.kwargs(ctx)
     )
 
     # Find a paginator that can describe this collection
