@@ -33,15 +33,7 @@ def create(
         botocore_session=None
     )
     resource: boto3.resources.factory.ResourceFactory = session.resource(
-        service_name=service_name,
-        region_name=ctx.acct.get("region_name"),
-        api_version=ctx.acct.get("api_version"),
-        use_ssl=ctx.acct.get("use_ssl", True),
-        endpoint_url=ctx.acct.get("endpoint_url"),
-        aws_access_key_id=ctx.acct.get("aws_access_key_id"),
-        aws_secret_access_key=ctx.acct.get("aws_secret_access_key"),
-        aws_session_token=ctx.acct.get("aws_session_token"),
-        verify=ctx.acct.get("verify"),
+        service_name=service_name, **hub.tool.boto3.session.kwargs(ctx)
     )
 
     # Don't pass kwargs that have a "None" value to the function call
@@ -139,15 +131,7 @@ async def id_(hub, ctx, service_name: str, collection: str, **kwargs) -> str:
     )
 
     resource: boto3.resources.factory.ResourceFactory = session.resource(
-        service_name=service_name,
-        region_name=ctx.acct.get("region_name"),
-        api_version=ctx.acct.get("api_version"),
-        use_ssl=ctx.acct.get("use_ssl", True),
-        endpoint_url=ctx.acct.get("endpoint_url"),
-        aws_access_key_id=ctx.acct.get("aws_access_key_id"),
-        aws_secret_access_key=ctx.acct.get("aws_secret_access_key"),
-        aws_session_token=ctx.acct.get("aws_session_token"),
-        verify=ctx.acct.get("verify"),
+        service_name=service_name, **hub.tool.boto3.session.kwargs(ctx)
     )
 
     assert any(
